@@ -1,6 +1,6 @@
 <html>
 <head>
-<meta http-equiv="refresh" content="0;url=packages.html">
+<meta http-equiv="refresh" content="0;url=classes.html">
 </head>
 <body>
 <?cs include:"analytics.cs" ?>
