@@ -22,16 +22,21 @@ import com.sun.javadoc.ClassDoc;
 import java.util.ArrayDeque;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.LinkedHashSet;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Queue;
 import java.util.Set;
 import java.util.TreeMap;
+import java.util.function.Predicate;
 
 public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Scoped, Resolvable {
 
@@ -135,6 +140,7 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     mIsPrimitive = isPrimitive;
     mAnnotations = annotations;
     mShowAnnotations = AnnotationInstanceInfo.getShowAnnotationsIntersection(annotations);
+    mHideAnnotations = AnnotationInstanceInfo.getHideAnnotationsIntersection(annotations);
   }
 
   public void init(TypeInfo typeInfo, ArrayList<ClassInfo> interfaces,
@@ -167,6 +173,7 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     mRealSuperclassType = superclassType;
     mAnnotations = annotations;
     mShowAnnotations = AnnotationInstanceInfo.getShowAnnotationsIntersection(annotations);
+    mHideAnnotations = AnnotationInstanceInfo.getHideAnnotationsIntersection(annotations);
 
     // after providing new methods and new superclass info,clear any cached
     // lists of self + superclass methods, ctors, etc.
@@ -197,6 +204,29 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     mRealInnerClasses = realInnerClasses;
   }
 
+  public class ClassMemberInfo extends MemberInfo {
+    public ClassMemberInfo() {
+      super(ClassInfo.this.getRawCommentText(), ClassInfo.this.name(), ClassInfo.this.name(),
+          ClassInfo.this, ClassInfo.this, ClassInfo.this.isPublic(), ClassInfo.this.isProtected(),
+          ClassInfo.this.isPackagePrivate(), ClassInfo.this.isPrivate(), ClassInfo.this.isFinal(),
+          ClassInfo.this.isStatic(), false, ClassInfo.this.kind(), ClassInfo.this.position(),
+          ClassInfo.this.annotations());
+    }
+
+    @Override
+    public boolean isExecutable() {
+      return false;
+    }
+  }
+
+  /**
+   * Return representation of this class as {@link MemberInfo}. This normally
+   * doesn't make any sense, but it's useful for {@link Predicate} testing.
+   */
+  public MemberInfo asMemberInfo() {
+    return new ClassMemberInfo();
+  }
+
   public ArrayList<ClassInfo> getRealInnerClasses() {
     return mRealInnerClasses;
   }
@@ -312,6 +342,19 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     return result;
   }
 
+  public TypeInfo getTypeParameter(String qualifiedTypeName) {
+      List<TypeInfo> parameters = mTypeInfo.typeArguments();
+      if (parameters == null) {
+          return null;
+      }
+      for (TypeInfo parameter : parameters) {
+          if (parameter.qualifiedTypeName().equals(qualifiedTypeName)) {
+              return parameter;
+          }
+      }
+      return null;
+  }
+
   /**
    * List of only direct interface's classes, without worrying about type param mapping.
    * This can't be lazy loaded, because its overloads depend on changing type parameters
@@ -556,9 +599,16 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
         }
       }
 
-      if (commentDeprecated != annotationDeprecated) {
+      // Check to see that the JavaDoc contains @deprecated AND the method is marked as @Deprecated.
+      // Otherwise, warn.
+      // Note: We only do this for "included" classes (i.e. those we have source code for); we do
+      // not have comments for classes from .class files but we do know whether a class is marked
+      // as @Deprecated.
+      if (isIncluded() && !isHiddenOrRemoved() && commentDeprecated != annotationDeprecated) {
         Errors.error(Errors.DEPRECATION_MISMATCH, position(), "Class " + qualifiedName()
-            + ": @Deprecated annotation and @deprecated comment do not match");
+            + ": @Deprecated annotation (" + (annotationDeprecated ? "" : "not ")
+            + "present) and @deprecated doc tag (" + (commentDeprecated ? "" : "not ")
+            + "present) do not match");
       }
 
       mIsDeprecated = commentDeprecated | annotationDeprecated;
@@ -718,6 +768,13 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
         }
       }
 
+      for (MethodInfo mi : annotationElements()) {
+        if (!mi.isHiddenOrRemoved()) {
+          // add annotation element as a field
+          methods.put(mi.name() + mi.signature(), mi);
+        }
+      }
+
       // sort it
       mSelfMethods = new ArrayList<MethodInfo>(methods.values());
       Collections.sort(mSelfMethods, MethodInfo.comparator);
@@ -737,78 +794,20 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     mRemovedMethods = Collections.unmodifiableList(removedMethods);
   }
 
-  /**
-   * @param allMethods all methods regardless of access levels. Selects the
-   * removed, public/protected ones and store them. If a class is removed, all its members
-   * are removed, even if the member may not have a @removed tag.
-   */
-  public void setRemovedSelfMethods(List<MethodInfo> allMethods) {
-    List<MethodInfo> removedSelfMethods = new ArrayList<MethodInfo>();
-    for (MethodInfo method : allMethods) {
-      if ((this.isRemoved() || method.isRemoved()) && (method.isPublic() || method.isProtected()) &&
-          (this.isPublic() || this.isProtected()) &&
-          (method.findOverriddenMethod(method.name(), method.signature()) == null)) {
-        removedSelfMethods.add(method);
-      }
-    }
-
-    Collections.sort(removedSelfMethods, MethodInfo.comparator);
-    mRemovedSelfMethods = Collections.unmodifiableList(removedSelfMethods);
+  public void setExhaustiveConstructors(List<MethodInfo> constructors) {
+    mExhaustiveConstructors = constructors;
   }
 
-  /**
-   * @param allCtors all constructors regardless of access levels.
-   * But only the public/protected removed constructors will be stored by the method.
-   * Removed constructors should never be deleted from source code because
-   * they were once public API.
-   */
-  public void setRemovedConstructors(List<MethodInfo> allCtors) {
-    List<MethodInfo> ctors = new ArrayList<MethodInfo>();
-    for (MethodInfo ctor : allCtors) {
-      if ((this.isRemoved() || ctor.isRemoved()) && (ctor.isPublic() || ctor.isProtected()) &&
-          (this.isPublic() || this.isProtected())) {
-        ctors.add(ctor);
-      }
-    }
-
-    Collections.sort(ctors, MethodInfo.comparator);
-    mRemovedConstructors = Collections.unmodifiableList(ctors);
+  public void setExhaustiveMethods(List<MethodInfo> methods) {
+    mExhaustiveMethods = methods;
   }
 
-  /**
-   * @param allFields all fields regardless of access levels.  Selects the
-   * removed, public/protected ones and store them. If a class is removed, all its members
-   * are removed, even if the member may not have a @removed tag.
-   */
-  public void setRemovedSelfFields(List<FieldInfo> allFields) {
-    List<FieldInfo> fields = new ArrayList<FieldInfo>();
-    for (FieldInfo field : allFields) {
-      if ((this.isRemoved() || field.isRemoved()) && (field.isPublic() || field.isProtected()) &&
-          (this.isPublic() || this.isProtected())) {
-        fields.add(field);
-      }
-    }
-
-    Collections.sort(fields, FieldInfo.comparator);
-    mRemovedSelfFields = Collections.unmodifiableList(fields);
+  public void setExhaustiveEnumConstants(List<FieldInfo> enumConstants) {
+    mExhaustiveEnumConstants = enumConstants;
   }
 
-  /**
-   * @param allEnumConstants all enum constants regardless of access levels. Selects the
-   * removed, public/protected ones and store them. If a class is removed, all its members
-   * are removed, even if the member may not have a @removed tag.
-   */
-  public void setRemovedEnumConstants(List<FieldInfo> allEnumConstants) {
-    List<FieldInfo> enums = new ArrayList<FieldInfo>();
-    for (FieldInfo field : allEnumConstants) {
-      if ((this.isRemoved() || field.isRemoved()) && (field.isPublic() || field.isProtected()) &&
-          (this.isPublic() || this.isProtected())) {
-        enums.add(field);
-      }
-    }
-
-    Collections.sort(enums, FieldInfo.comparator);
-    mRemovedEnumConstants = Collections.unmodifiableList(enums);
+  public void setExhaustiveFields(List<FieldInfo> fields) {
+    mExhaustiveFields = fields;
   }
 
   /**
@@ -819,58 +818,128 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     return mRemovedMethods;
   }
 
-  /**
-   * @return all public/protected methods that are removed. @removed methods should never be
-   * deleted from source code because they were once public API. Methods that override
-   * a parent method will not be included, because deleting them does not break the API.
-   */
-  public List<MethodInfo> getRemovedSelfMethods() {
-    return mRemovedSelfMethods;
+  public List<MethodInfo> getExhaustiveConstructors() {
+    return mExhaustiveConstructors;
+  }
+
+  public List<MethodInfo> getExhaustiveMethods() {
+    return mExhaustiveMethods;
+  }
+
+  public List<FieldInfo> getExhaustiveEnumConstants() {
+    return mExhaustiveEnumConstants;
+  }
+
+  public List<FieldInfo> getExhaustiveFields() {
+    return mExhaustiveFields;
   }
 
   /**
-   * @return all public constructors that are removed.
-   * removed constructors should never be deleted from source code because they
-   * were once public API.
-   * The returned list is sorted and unmodifiable.
+   * Return list of ancestor classes that contribute to this class through
+   * inheritance. Ordered from most general to most specific with all interfaces
+   * listed before concrete classes.
    */
-  public List<MethodInfo> getRemovedConstructors() {
-    return mRemovedConstructors;
+  public List<ClassInfo> gatherAncestorClasses() {
+    LinkedList<ClassInfo> classes = gatherAncestorClasses(new LinkedList<>());
+    classes.removeLast();
+    return classes;
   }
 
-  /**
-   * @return all public/protected fields that are removed.
-   * removed members should never be deleted from source code because they were once public API.
-   * The returned list is sorted and unmodifiable.
-   */
-  public List<FieldInfo> getRemovedSelfFields() {
-    return mRemovedSelfFields;
-  }
-
-  /**
-   * @return all public/protected enumConstants that are removed.
-   * removed members should never be deleted from source code
-   * because they were once public API.
-   * The returned list is sorted and unmodifiable.
-   */
-  public List<FieldInfo> getRemovedSelfEnumConstants() {
-    return mRemovedEnumConstants;
-  }
-
-  /**
-   * @return true if this class contains any self members that are removed
-   */
-  public boolean hasRemovedSelfMembers() {
-    List<FieldInfo> removedSelfFields = getRemovedSelfFields();
-    List<FieldInfo> removedSelfEnumConstants = getRemovedSelfEnumConstants();
-    List<MethodInfo> removedSelfMethods = getRemovedSelfMethods();
-    List<MethodInfo> removedConstructors = getRemovedConstructors();
-    if (removedSelfFields.size() + removedSelfEnumConstants.size()
-        + removedSelfMethods.size() + removedConstructors.size() == 0) {
-      return false;
-    } else {
-      return true;
+  private LinkedList<ClassInfo> gatherAncestorClasses(LinkedList<ClassInfo> classes) {
+    classes.add(0, this);
+    if (mRealSuperclass != null) {
+      mRealSuperclass.gatherAncestorClasses(classes);
     }
+    if (mRealInterfaces != null) {
+      for (ClassInfo clazz : mRealInterfaces) {
+        clazz.gatherAncestorClasses(classes);
+      }
+    }
+    return classes;
+  }
+
+  /**
+   * Return superclass matching the given predicate. When a superclass doesn't
+   * match, we'll keep crawling up the tree until we find someone who matches.
+   */
+  public ClassInfo filteredSuperclass(Predicate<MemberInfo> predicate) {
+    if (mRealSuperclass == null) {
+      return null;
+    } else if (predicate.test(mRealSuperclass.asMemberInfo())) {
+      return mRealSuperclass;
+    } else {
+      return mRealSuperclass.filteredSuperclass(predicate);
+    }
+  }
+
+  /**
+   * Return interfaces matching the given predicate. When a superclass or
+   * interface doesn't match, we'll keep crawling up the tree until we find
+   * someone who matches.
+   */
+  public Collection<ClassInfo> filteredInterfaces(Predicate<MemberInfo> predicate) {
+    return filteredInterfaces(predicate, new LinkedHashSet<>());
+  }
+
+  private LinkedHashSet<ClassInfo> filteredInterfaces(Predicate<MemberInfo> predicate,
+      LinkedHashSet<ClassInfo> classes) {
+    if (mRealSuperclass != null && !predicate.test(mRealSuperclass.asMemberInfo())) {
+      mRealSuperclass.filteredInterfaces(predicate, classes);
+    }
+    if (mRealInterfaces != null) {
+      for (ClassInfo clazz : mRealInterfaces) {
+        if (predicate.test(clazz.asMemberInfo())) {
+          classes.add(clazz);
+        } else {
+          clazz.filteredInterfaces(predicate, classes);
+        }
+      }
+    }
+    return classes;
+  }
+
+  /**
+   * Return methods matching the given predicate. Forcibly includes local
+   * methods that override a matching method in an ancestor class.
+   */
+  public Collection<MethodInfo> filteredMethods(Predicate<MemberInfo> predicate) {
+    Set<MethodInfo> methods = new LinkedHashSet<>();
+    for (MethodInfo method : getExhaustiveMethods()) {
+      if (predicate.test(method) || (method.findPredicateOverriddenMethod(predicate) != null)) {
+        methods.remove(method);
+        methods.add(method);
+      }
+    }
+    return methods;
+  }
+
+  /**
+   * Return fields matching the given predicate. Also clones fields from
+   * ancestors that would match had they been defined in this class.
+   */
+  public Collection<FieldInfo> filteredFields(Predicate<MemberInfo> predicate) {
+    Set<FieldInfo> fields = new LinkedHashSet<>();
+    if (Doclava.showUnannotated) {
+      for (ClassInfo clazz : gatherAncestorClasses()) {
+        if (!clazz.isInterface()) continue;
+        for (FieldInfo field : clazz.getExhaustiveFields()) {
+          if (!predicate.test(field)) {
+            field = field.cloneForClass(this);
+            if (predicate.test(field)) {
+              fields.remove(field);
+              fields.add(field);
+            }
+          }
+        }
+      }
+    }
+    for (FieldInfo field : getExhaustiveFields()) {
+      if (predicate.test(field)) {
+        fields.remove(field);
+        fields.add(field);
+      }
+    }
+    return fields;
   }
 
   public void addMethod(MethodInfo method) {
@@ -1190,6 +1259,7 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     if (isDeprecated()) {
       data.setValue(base + ".deprecatedsince", getDeprecatedSince());
     }
+    data.setValue(base + ".artifact", getArtifact());
 
     ArrayList<AnnotationInstanceInfo> showAnnos = getShowAnnotationsIncludeOuters();
     AnnotationInstanceInfo.makeLinkListHDF(
@@ -1260,6 +1330,7 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     if (isDeprecated()) {
       data.setValue("class.deprecatedsince", getDeprecatedSince());
     }
+    data.setValue("class.artifact", getArtifact());
     setFederatedReferences(data, "class");
 
     // the containing package -- note that this can be passed to type_link,
@@ -1285,13 +1356,14 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
 
     // class description
     TagInfo.makeHDF(data, "class.descr", inlineTags());
+    TagInfo.makeHDF(data, "class.descrAux", Doclava.auxSource.classAuxTags(this));
     TagInfo.makeHDF(data, "class.seeAlso", comment().seeTags());
     TagInfo.makeHDF(data, "class.deprecated", deprecatedTags());
 
     // known subclasses
     TreeMap<String, ClassInfo> direct = new TreeMap<String, ClassInfo>();
     TreeMap<String, ClassInfo> indirect = new TreeMap<String, ClassInfo>();
-    ClassInfo[] all = Converter.rootClasses();
+    Collection<ClassInfo> all = Converter.rootClasses();
     for (ClassInfo cl : all) {
       if (cl.superclass() != null && cl.superclass().equals(this)) {
         direct.put(cl.name(), cl);
@@ -1488,7 +1560,7 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     if (ctp.classInfo().mIsIncluded) {
       data.setValue(base + ".included", "true");
     } else {
-      Doclava.federationTagger.tagAll(new ClassInfo[] {ctp.classInfo()});
+      Doclava.federationTagger.tagAll(Arrays.asList(ctp.classInfo()));
       if (!ctp.classInfo().getFederatedReferences().isEmpty()) {
         FederatedSite site = ctp.classInfo().getFederatedReferences().iterator().next();
         data.setValue(base + ".link", site.linkFor(ctp.classInfo().htmlPage()));
@@ -1539,8 +1611,8 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
   }
 
   /**
-   * @return true if the containing package has @hide comment, or an ancestor
-   * class of this class is hidden, or this class has @hide comment.
+   * @return true if the containing package has @hide comment, a hide annotaion,
+   * or a containing class of this class is hidden.
    */
   public boolean isHiddenImpl() {
     ClassInfo cl = this;
@@ -1552,7 +1624,7 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
       if (pkg != null && pkg.hasHideComment()) {
         return true;
       }
-      if (cl.comment().isHidden()) {
+      if (cl.comment().isHidden() || cl.hasHideAnnotation()) {
         return true;
       }
       cl = cl.containingClass();
@@ -1576,9 +1648,6 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
   public boolean isRemovedImpl() {
     ClassInfo cl = this;
     while (cl != null) {
-      if (cl.hasShowAnnotation()) {
-        return false;
-      }
       PackageInfo pkg = cl.containingPackage();
       if (pkg != null && pkg.hasRemovedComment()) {
         return true;
@@ -1604,6 +1673,14 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     return mShowAnnotations;
   }
 
+  public boolean hasHideAnnotation() {
+    return mHideAnnotations != null && mHideAnnotations.size() > 0;
+  }
+
+  public ArrayList<AnnotationInstanceInfo> hideAnnotations() {
+    return mHideAnnotations;
+  }
+
   public ArrayList<AnnotationInstanceInfo> getShowAnnotationsIncludeOuters() {
     ArrayList<AnnotationInstanceInfo> allAnnotations = new ArrayList<AnnotationInstanceInfo>();
     ClassInfo cl = this;
@@ -1778,14 +1855,6 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     return rv;
   }
 
-  public boolean equals(ClassInfo that) {
-    if (that != null) {
-      return this.qualifiedName().equals(that.qualifiedName());
-    } else {
-      return false;
-    }
-  }
-
   public void setNonWrittenConstructors(ArrayList<MethodInfo> nonWritten) {
     mNonWrittenConstructors = nonWritten;
   }
@@ -1838,6 +1907,23 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
     return this.qualifiedName();
   }
 
+  @Override
+  public boolean equals(Object o) {
+    if (this == o) {
+      return true;
+    } else if (o instanceof ClassInfo) {
+      final ClassInfo c = (ClassInfo) o;
+      return mQualifiedName.equals(c.mQualifiedName);
+    } else {
+      return false;
+    }
+  }
+
+  @Override
+  public int hashCode() {
+    return mQualifiedName.hashCode();
+  }
+
   public void setReasonIncluded(String reason) {
     mReasonIncluded = reason;
   }
@@ -1891,6 +1977,7 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
   private ClassInfo mSuperclass;
   private ArrayList<AnnotationInstanceInfo> mAnnotations;
   private ArrayList<AnnotationInstanceInfo> mShowAnnotations;
+  private ArrayList<AnnotationInstanceInfo> mHideAnnotations;
   private boolean mSuperclassInit;
   private boolean mDeprecatedKnown;
 
@@ -1923,12 +2010,12 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
   // Resolutions
   private ArrayList<Resolution> mResolutions;
 
-  private List<MethodInfo> mRemovedConstructors; // immutable after you set its value.
-  // @removed self methods that do not override any parent methods
-  private List<MethodInfo> mRemovedSelfMethods; // immutable after you set its value.
   private List<MethodInfo> mRemovedMethods; // immutable after you set its value.
-  private List<FieldInfo> mRemovedSelfFields; // immutable after you set its value.
-  private List<FieldInfo> mRemovedEnumConstants; // immutable after you set its value.
+
+  private List<MethodInfo> mExhaustiveConstructors; // immutable after you set its value.
+  private List<MethodInfo> mExhaustiveMethods; // immutable after you set its value.
+  private List<FieldInfo> mExhaustiveEnumConstants; // immutable after you set its value.
+  private List<FieldInfo> mExhaustiveFields; // immutable after you set its value.
 
   /**
    * Returns true if {@code cl} implements the interface {@code iface} either by either being that
@@ -2086,8 +2173,13 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
           mi = ClassInfo.interfaceMethod(mInfo, cl);
         }
         if (mi == null) {
-          Errors.error(Errors.REMOVED_METHOD, mInfo.position(), "Removed public method "
-              + mInfo.prettyQualifiedSignature());
+          if (mInfo.isDeprecated()) {
+            Errors.error(Errors.REMOVED_DEPRECATED_METHOD, mInfo.position(),
+                "Removed deprecated public method " + mInfo.prettyQualifiedSignature());
+          } else {
+            Errors.error(Errors.REMOVED_METHOD, mInfo.position(),
+                "Removed public method " + mInfo.prettyQualifiedSignature());
+          }
           consistent = false;
         }
       }
@@ -2101,16 +2193,20 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
          * abstractness affects how users use it. See also Stubs.methodIsOverride().
          */
         MethodInfo mi = ClassInfo.overriddenMethod(mInfo, this);
-        if (mi == null ||
-            mi.isAbstract() != mInfo.isAbstract()) {
-          Errors.error(Errors.ADDED_METHOD, mInfo.position(), "Added public method "
-              + mInfo.prettyQualifiedSignature());
-          if (diffMode) {
-            newMethods.add(mInfo);
-          }
+        if (mi == null && mInfo.isAbstract()) {
+          Errors.error(Errors.ADDED_ABSTRACT_METHOD, mInfo.position(),
+              "Added abstract public method "
+              + mInfo.prettyQualifiedSignature() + " to existing class");
           consistent = false;
+        } else if (mi == null || mi.isAbstract() != mInfo.isAbstract()) {
+            Errors.error(Errors.ADDED_METHOD, mInfo.position(), "Added public method "
+                + mInfo.prettyQualifiedSignature());
+            if (diffMode) {
+              newMethods.add(mInfo);
+            }
+            consistent = false;
+          }
         }
-      }
     }
     if (diffMode) {
       Collections.sort(newMethods, MethodInfo.comparator);
@@ -2122,8 +2218,13 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
           consistent = false;
         }
       } else {
-        Errors.error(Errors.REMOVED_METHOD, mInfo.position(), "Removed public constructor "
-            + mInfo.prettyQualifiedSignature());
+        if (mInfo.isDeprecated()) {
+          Errors.error(Errors.REMOVED_DEPRECATED_METHOD, mInfo.position(),
+              "Removed deprecated public constructor " + mInfo.prettyQualifiedSignature());
+        } else {
+          Errors.error(Errors.REMOVED_METHOD, mInfo.position(),
+              "Removed public constructor " + mInfo.prettyQualifiedSignature());
+        }
         consistent = false;
       }
     }
@@ -2147,8 +2248,13 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
           consistent = false;
         }
       } else {
-        Errors.error(Errors.REMOVED_FIELD, mInfo.position(), "Removed field "
-            + mInfo.qualifiedName());
+        if (mInfo.isDeprecated()) {
+          Errors.error(Errors.REMOVED_DEPRECATED_FIELD, mInfo.position(),
+              "Removed deprecated field " + mInfo.qualifiedName());
+        } else {
+          Errors.error(Errors.REMOVED_FIELD, mInfo.position(),
+              "Removed field " + mInfo.qualifiedName());
+        }
         consistent = false;
       }
     }
@@ -2166,8 +2272,13 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
           consistent = false;
         }
       } else {
-        Errors.error(Errors.REMOVED_FIELD, info.position(), "Removed enum constant "
-            + info.qualifiedName());
+        if (info.isDeprecated()) {
+          Errors.error(Errors.REMOVED_DEPRECATED_FIELD, info.position(),
+              "Removed deprecated enum constant " + info.qualifiedName());
+        } else {
+          Errors.error(Errors.REMOVED_FIELD, info.position(),
+              "Removed enum constant " + info.qualifiedName());
+        }
         consistent = false;
       }
     }
@@ -2232,6 +2343,17 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
       }
     }
 
+    if (hasTypeParameters() && cl.hasTypeParameters()) {
+      ArrayList<TypeInfo> oldParams = typeParameters();
+      ArrayList<TypeInfo> newParams = cl.typeParameters();
+      if (oldParams.size() != newParams.size()) {
+        consistent = false;
+        Errors.error(Errors.CHANGED_TYPE, cl.position(), "Class " + qualifiedName()
+            + " changed number of type parameters from " + oldParams.size()
+            + " to " + newParams.size());
+      }
+    }
+
     return consistent;
   }
 
@@ -2284,6 +2406,20 @@ public class ClassInfo extends DocInfo implements ContainerInfo, Comparable, Sco
       return mTypeInfo;
   }
 
+  public boolean hasTypeParameters() {
+      if (mTypeInfo != null && mTypeInfo.typeArguments() != null) {
+          return !mTypeInfo.typeArguments().isEmpty();
+      }
+      return false;
+  }
+
+  public ArrayList<TypeInfo> typeParameters() {
+      if (hasTypeParameters()) {
+          return mTypeInfo.typeArguments();
+      }
+      return null;
+  }
+
   public void addInnerClass(ClassInfo innerClass) {
       if (mInnerClasses == null) {
           mInnerClasses = new ArrayList<ClassInfo>();
