@@ -28,6 +28,7 @@ public class Comment {
       Pattern.compile("((.*?)\\.)[ \t\r\n\\<](.*)", Pattern.DOTALL);
 
   private static final Set<String> KNOWN_TAGS = new HashSet<String>(Arrays.asList(new String[] {
+          "@apiNote",
           "@author",
           "@since",
           "@version",
@@ -41,6 +42,9 @@ public class Comment {
           "@sample",
           "@include",
           "@serial",
+          "@implNote",
+          "@implSpec",
+          "@usesMathJax",
       }));
 
   public Comment(String text, ContainerInfo base, SourcePositionInfo sp) {
@@ -164,9 +168,15 @@ public class Comment {
   }
 
   private int findEndIndexOfInlineTag(String text, int fromIndex, int toIndex) {
+      int braceDepth = 0;
       for (int i = fromIndex; i < toIndex; i++) {
-          if (text.charAt(i) == '}') {
-              return i;
+          if (text.charAt(i) == '{') {
+              braceDepth++;
+          } else if (text.charAt(i) == '}') {
+              braceDepth--;
+              if (braceDepth == 0) {
+                  return i;
+              }
           }
       }
 
@@ -321,7 +331,12 @@ public class Comment {
     } else if (name.equals("@see")) {
       mSeeTagsList.add(new SeeTagInfo("@see", "@see", text, mBase, pos));
     } else if (name.equals("@link")) {
-      mInlineTagsList.add(new SeeTagInfo(name, "@see", text, mBase, pos));
+      if (Doclava.DEVSITE_IGNORE_JDLINKS) {
+        TagInfo linkTag = new TextTagInfo(name, name, text, pos);
+        mInlineTagsList.add(linkTag);
+      } else {
+        mInlineTagsList.add(new SeeTagInfo(name, "@see", text, mBase, pos));
+      }
     } else if (name.equals("@linkplain")) {
       mInlineTagsList.add(new SeeTagInfo(name, "@linkplain", text, mBase, pos));
     } else if (name.equals("@value")) {
@@ -356,6 +371,14 @@ public class Comment {
       mUndeprecateTagsList.add(new TextTagInfo("@undeprecate", "@undeprecate", text, pos));
     } else if (name.equals("@include") || name.equals("@sample")) {
       mInlineTagsList.add(new SampleTagInfo(name, "@include", text, mBase, pos));
+    } else if (name.equals("@apiNote") || name.equals("@implSpec") || name.equals("@implNote")) {
+      mTagsList.add(new ParsedTagInfo(name, name, text, mBase, pos));
+    } else if (name.equals("@memberDoc")) {
+      mMemberDocTagsList.add(new ParsedTagInfo("@memberDoc", "@memberDoc", text, mBase, pos));
+    } else if (name.equals("@paramDoc")) {
+      mParamDocTagsList.add(new ParsedTagInfo("@paramDoc", "@paramDoc", text, mBase, pos));
+    } else if (name.equals("@returnDoc")) {
+      mReturnDocTagsList.add(new ParsedTagInfo("@returnDoc", "@returnDoc", text, mBase, pos));
     } else {
       boolean known = KNOWN_TAGS.contains(name);
       if (!known) {
@@ -424,6 +447,11 @@ public class Comment {
     return results.toArray(TagInfo.getArray(results.size()));
   }
 
+  public TagInfo[] blockTags() {
+    init();
+    return mTags;
+  }
+
   public ParamTagInfo[] paramTags() {
     init();
     return mParamTags;
@@ -464,6 +492,21 @@ public class Comment {
     return mBriefTags;
   }
 
+  public ParsedTagInfo[] memberDocTags() {
+    init();
+    return mMemberDocTags;
+  }
+
+  public ParsedTagInfo[] paramDocTags() {
+    init();
+    return mParamDocTags;
+  }
+
+  public ParsedTagInfo[] returnDocTags() {
+    init();
+    return mReturnDocTags;
+  }
+
   public boolean isHidden() {
     if (mHidden == null) {
       mHidden = !Doclava.checkLevel(Doclava.SHOW_HIDDEN) &&
@@ -522,6 +565,7 @@ public class Comment {
     mInitialized = true;
 
     mInlineTags = mInlineTagsList.toArray(TagInfo.getArray(mInlineTagsList.size()));
+    mTags = mTagsList.toArray(TagInfo.getArray(mTagsList.size()));
     mParamTags = mParamTagsList.toArray(ParamTagInfo.getArray(mParamTagsList.size()));
     mSeeTags = mSeeTagsList.toArray(SeeTagInfo.getArray(mSeeTagsList.size()));
     mThrowsTags = mThrowsTagsList.toArray(ThrowsTagInfo.getArray(mThrowsTagsList.size()));
@@ -532,7 +576,11 @@ public class Comment {
     mUndeprecateTags = mUndeprecateTagsList.toArray(TagInfo.getArray(mUndeprecateTagsList.size()));
     mAttrTags = mAttrTagsList.toArray(AttrTagInfo.getArray(mAttrTagsList.size()));
     mBriefTags = mBriefTagsList.toArray(TagInfo.getArray(mBriefTagsList.size()));
+    mMemberDocTags = mMemberDocTagsList.toArray(ParsedTagInfo.getArray(mMemberDocTagsList.size()));
+    mParamDocTags = mParamDocTagsList.toArray(ParsedTagInfo.getArray(mParamDocTagsList.size()));
+    mReturnDocTags = mReturnDocTagsList.toArray(ParsedTagInfo.getArray(mReturnDocTagsList.size()));
 
+    mTagsList = null;
     mParamTagsList = null;
     mSeeTagsList = null;
     mThrowsTagsList = null;
@@ -541,6 +589,9 @@ public class Comment {
     mUndeprecateTagsList = null;
     mAttrTagsList = null;
     mBriefTagsList = null;
+    mMemberDocTagsList = null;
+    mParamDocTagsList = null;
+    mReturnDocTagsList = null;
   }
 
   boolean mInitialized;
@@ -563,6 +614,9 @@ public class Comment {
   TagInfo[] mDeprecatedTags;
   TagInfo[] mUndeprecateTags;
   AttrTagInfo[] mAttrTags;
+  ParsedTagInfo[] mMemberDocTags;
+  ParsedTagInfo[] mParamDocTags;
+  ParsedTagInfo[] mReturnDocTags;
 
   ArrayList<TagInfo> mInlineTagsList = new ArrayList<TagInfo>();
   ArrayList<TagInfo> mTagsList = new ArrayList<TagInfo>();
@@ -574,6 +628,8 @@ public class Comment {
   ArrayList<ParsedTagInfo> mDeprecatedTagsList = new ArrayList<ParsedTagInfo>();
   ArrayList<TagInfo> mUndeprecateTagsList = new ArrayList<TagInfo>();
   ArrayList<AttrTagInfo> mAttrTagsList = new ArrayList<AttrTagInfo>();
-
+  ArrayList<ParsedTagInfo> mMemberDocTagsList = new ArrayList<ParsedTagInfo>();
+  ArrayList<ParsedTagInfo> mParamDocTagsList = new ArrayList<ParsedTagInfo>();
+  ArrayList<ParsedTagInfo> mReturnDocTagsList = new ArrayList<ParsedTagInfo>();
 
 }
