@@ -16,7 +16,7 @@
 
 package com.google.doclava;
 
-
+import com.google.doclava.apicheck.ApiInfo;
 import com.sun.javadoc.AnnotationDesc;
 import com.sun.javadoc.AnnotationTypeDoc;
 import com.sun.javadoc.AnnotationTypeElementDoc;
@@ -39,15 +39,19 @@ import com.sun.javadoc.Type;
 
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
 
 public class Converter {
   private static RootDoc root;
+  private static List<ApiInfo> apis;
 
   public static void makeInfo(RootDoc r) {
     root = r;
+    apis = new ArrayList<>();
 
     // create the objects
     ClassDoc[] classes = getClasses(r);
@@ -78,6 +82,15 @@ public class Converter {
     mRootClasses = Converter.convertClasses(classes);
   }
 
+  /**
+   * Adds additional APIs to be available from calls to obtain() methods.
+   *
+   * @param api the APIs to add, must be non-{@code null}
+   */
+  public static void addApiInfo(ApiInfo api) {
+    apis.add(api);
+  }
+
   private static ClassDoc[] getClasses(RootDoc r) {
     ClassDoc[] classDocs = r.classes();
     ArrayList<ClassDoc> filtered = new ArrayList<ClassDoc>(classDocs.length);
@@ -99,12 +112,12 @@ public class Converter {
 
   private static ClassInfo[] mRootClasses;
 
-  public static ClassInfo[] rootClasses() {
-    return mRootClasses;
+  public static Collection<ClassInfo> rootClasses() {
+    return Arrays.asList(mRootClasses);
   }
 
-  public static ClassInfo[] allClasses() {
-    return (ClassInfo[]) mClasses.all();
+  public static Collection<ClassInfo> allClasses() {
+    return (Collection<ClassInfo>) mClasses.all();
   }
 
   private static final MethodDoc[] EMPTY_METHOD_DOC = new MethodDoc[0];
@@ -137,14 +150,14 @@ public class Converter {
     cl.setRemovedMethods(
             new ArrayList<MethodInfo>(Arrays.asList(Converter.getRemovedMethods(c.methods(false)))));
 
-    cl.setRemovedSelfMethods(
-        new ArrayList<MethodInfo>(Converter.convertAllMethods(c.methods(false))));
-    cl.setRemovedConstructors(
+    cl.setExhaustiveConstructors(
         new ArrayList<MethodInfo>(Converter.convertAllMethods(c.constructors(false))));
-    cl.setRemovedSelfFields(
-        new ArrayList<FieldInfo>(Converter.convertAllFields(c.fields(false))));
-    cl.setRemovedEnumConstants(
+    cl.setExhaustiveMethods(
+        new ArrayList<MethodInfo>(Converter.convertAllMethods(c.methods(false))));
+    cl.setExhaustiveEnumConstants(
         new ArrayList<FieldInfo>(Converter.convertAllFields(c.enumConstants())));
+    cl.setExhaustiveFields(
+        new ArrayList<FieldInfo>(Converter.convertAllFields(c.fields(false))));
 
     cl.setNonWrittenConstructors(
             new ArrayList<MethodInfo>(Arrays.asList(Converter.convertNonWrittenConstructors(
@@ -155,12 +168,54 @@ public class Converter {
                     Converter.convertClasses(c.innerClasses(false)))));
   }
 
+  /**
+   * Obtains a {@link ClassInfo} describing the specified class. If the class
+   * was not specified on the source path, this method will attempt to locate
+   * the class within the list of federated APIs.
+   *
+   * @param className the fully-qualified class name to search for
+   * @return info for the specified class, or {@code null} if not available
+   * @see #addApiInfo(ApiInfo)
+   */
   public static ClassInfo obtainClass(String className) {
-    return Converter.obtainClass(root.classNamed(className));
+    ClassInfo result = Converter.obtainClass(root.classNamed(className));
+    if (result != null) {
+      return result;
+    }
+
+    for (ApiInfo api : apis) {
+      result = api.findClass(className);
+      if (result != null) {
+        return result;
+      }
+    }
+
+    return null;
   }
 
+  /**
+   * Obtains a {@link PackageInfo} describing the specified package. If the
+   * package was not specified on the source path, this method will attempt to
+   * locate the package within the list of federated APIs.
+   *
+   * @param packageName the fully-qualified package name to search for
+   * @return info for the specified package, or {@code null} if not available
+   * @see #addApiInfo(ApiInfo)
+   */
   public static PackageInfo obtainPackage(String packageName) {
-    return Converter.obtainPackage(root.packageNamed(packageName));
+    PackageInfo result = Converter.obtainPackage(root.packageNamed(packageName));
+    if (result != null) {
+      return result;
+    }
+
+    for (ApiInfo api : apis) {
+      result = api.getPackages().get(packageName);
+      if (result != null) {
+        return result;
+      }
+    }
+
+    return null;
   }
 
   private static TagInfo convertTag(Tag tag) {
@@ -227,7 +282,8 @@ public class Converter {
     if (p == null) return null;
     ParameterInfo pi =
         new ParameterInfo(p.name(), p.typeName(), Converter.obtainType(p.type()), isVarArg,
-          Converter.convertSourcePosition(pos));
+          Converter.convertSourcePosition(pos),
+          Arrays.asList(Converter.convertAnnotationInstances(p.annotations())));
     return pi;
   }
 
@@ -296,8 +352,8 @@ public class Converter {
     }
 
     @Override
-    ClassInfo[] all() {
-      return mCache.values().toArray(new ClassInfo[mCache.size()]);
+    Collection<?> all() {
+      return mCache.values();
     }
   };
 
@@ -403,8 +459,9 @@ public class Converter {
                     m.name(), m.signature(), Converter.obtainClass(m.containingClass()),
                     Converter.obtainClass(m.containingClass()), m.isPublic(), m.isProtected(), m
                     .isPackagePrivate(), m.isPrivate(), m.isFinal(), m.isStatic(), m.isSynthetic(),
-                    m.isAbstract(), m.isSynchronized(), m.isNative(), true, "annotationElement",
-                    m.flatSignature(), Converter.obtainMethod(m.overriddenMethod()),
+                    m.isAbstract(), m.isSynchronized(), m.isNative(), m.isDefault(), true,
+                    "annotationElement", m.flatSignature(),
+                    Converter.obtainMethod(m.overriddenMethod()),
                     Converter.obtainType(m.returnType()),
                     new ArrayList<ParameterInfo>(Arrays.asList(
                             Converter.convertParameters(m.parameters(), m))),
@@ -417,15 +474,20 @@ public class Converter {
         return result;
       } else if (o instanceof MethodDoc) {
         MethodDoc m = (MethodDoc) o;
+        final ClassInfo containingClass = Converter.obtainClass(m.containingClass());
+
+        // The containing class is final, so it is implied that every method is final as well.
+        // No need to apply 'final' to each method.
+        final boolean isMethodFinal = m.isFinal() && !containingClass.isFinal();
         MethodInfo result =
             new MethodInfo(m.getRawCommentText(),
                     new ArrayList<TypeInfo>(Arrays.asList(
                             Converter.convertTypes(m.typeParameters()))), m.name(), m.signature(),
-                    Converter.obtainClass(m.containingClass()),
-                    Converter.obtainClass(m.containingClass()), m.isPublic(), m.isProtected(),
-                    m.isPackagePrivate(), m.isPrivate(), m.isFinal(), m.isStatic(), m.isSynthetic(),
-                    m.isAbstract(), m.isSynchronized(), m.isNative(), false, "method",
-                    m.flatSignature(), Converter.obtainMethod(m.overriddenMethod()),
+                    containingClass, containingClass, m.isPublic(), m.isProtected(),
+                    m.isPackagePrivate(), m.isPrivate(), isMethodFinal, m.isStatic(),
+                    m.isSynthetic(), m.isAbstract(), m.isSynchronized(), m.isNative(),
+                    m.isDefault(), false, "method", m.flatSignature(),
+                    Converter.obtainMethod(m.overriddenMethod()),
                     Converter.obtainType(m.returnType()),
                     new ArrayList<ParameterInfo>(Arrays.asList(
                             Converter.convertParameters(m.parameters(), m))),
@@ -439,12 +501,26 @@ public class Converter {
         return result;
       } else {
         ConstructorDoc m = (ConstructorDoc) o;
+        // Workaround for a JavaDoc behavior change introduced in OpenJDK 8 that breaks
+        // links in documentation and the content of API files like current.txt.
+        // http://b/18051133.
+        String name = m.name();
+        ClassDoc containingClass = m.containingClass();
+        if (containingClass.containingClass() != null) {
+          // This should detect the new behavior and be bypassed otherwise.
+          if (!name.contains(".")) {
+            // Constructors of inner classes do not contain the name of the enclosing class
+            // with OpenJDK 8. This simulates the old behavior:
+            name = containingClass.name();
+          }
+        }
+        // End of workaround.
         MethodInfo result =
-            new MethodInfo(m.getRawCommentText(), new ArrayList<TypeInfo>(Arrays.asList(Converter.convertTypes(m.typeParameters()))), m
-                .name(), m.signature(), Converter.obtainClass(m.containingClass()), Converter
+            new MethodInfo(m.getRawCommentText(), new ArrayList<TypeInfo>(Arrays.asList(Converter.convertTypes(m.typeParameters()))),
+                name, m.signature(), Converter.obtainClass(m.containingClass()), Converter
                 .obtainClass(m.containingClass()), m.isPublic(), m.isProtected(), m
                 .isPackagePrivate(), m.isPrivate(), m.isFinal(), m.isStatic(), m.isSynthetic(),
-                false, m.isSynchronized(), m.isNative(), false, "constructor", m.flatSignature(),
+                false, m.isSynchronized(), m.isNative(), false/*isDefault*/, false, "constructor", m.flatSignature(),
                 null, null, new ArrayList<ParameterInfo>(Arrays.asList(Converter.convertParameters(m.parameters(), m))),
                 new ArrayList<ClassInfo>(Arrays.asList(Converter.convertClasses(m.thrownExceptions()))), Converter.convertSourcePosition(m
                     .position()), new ArrayList<AnnotationInstanceInfo>(Arrays.asList(Converter.convertAnnotationInstances(m.annotations()))));
@@ -544,7 +620,10 @@ public class Converter {
     @Override
     protected Object keyFor(Object o) {
       Type t = (Type) o;
-      String keyString = o.getClass().getName() + "/" + o.toString() + "/";
+      while (t.asAnnotatedType() != null) {
+        t = t.asAnnotatedType().underlyingType();
+      }
+      String keyString = t.getClass().getName() + "/" + t.toString() + "/";
       if (t.asParameterizedType() != null) {
         keyString += t.asParameterizedType().toString() + "/";
         if (t.asParameterizedType().typeArguments() != null) {
@@ -685,7 +764,7 @@ public class Converter {
       return o;
     }
 
-    Object[] all() {
+    Collection<?> all() {
       return null;
     }
   }
