@@ -28,6 +28,8 @@ import com.sun.javadoc.*;
 import java.util.*;
 import java.util.jar.JarFile;
 import java.util.regex.Matcher;
+import java.util.regex.Pattern;
+import java.util.stream.Collectors;
 import java.io.*;
 import java.lang.reflect.Proxy;
 import java.lang.reflect.Array;
@@ -44,7 +46,7 @@ public class Doclava {
   private static final String SDK_CONSTANT_TYPE_BROADCAST_ACTION =
       "android.annotation.SdkConstant.SdkConstantType.BROADCAST_INTENT_ACTION";
   private static final String SDK_CONSTANT_TYPE_SERVICE_ACTION =
-      "android.annotation.SdkConstant.SdkConstantType.SERVICE_INTENT_ACTION";
+      "android.annotation.SdkConstant.SdkConstantType.SERVICE_ACTION";
   private static final String SDK_CONSTANT_TYPE_CATEGORY =
       "android.annotation.SdkConstant.SdkConstantType.INTENT_CATEGORY";
   private static final String SDK_CONSTANT_TYPE_FEATURE =
@@ -68,13 +70,26 @@ public class Doclava {
   public static final boolean SORT_BY_NAV_GROUPS = true;
   /* Debug output for PageMetadata, format urls from site root */
   public static boolean META_DBG=false;
-
+  /* Generate the static html docs with devsite tempating only */
+  public static boolean DEVSITE_STATIC_ONLY = false;
+  /* Don't resolve @link refs found in devsite pages */
+  public static boolean DEVSITE_IGNORE_JDLINKS = false;
+  /* Show Preview navigation and process preview docs */
+  public static boolean INCLUDE_PREVIEW = false;
+  /* output en, es, ja without parent intl/ container */
+  public static boolean USE_DEVSITE_LOCALE_OUTPUT_PATHS = false;
+  /* generate navtree.js without other docs */
+  public static boolean NAVTREE_ONLY = false;
+  /* Generate reference navtree.js with all inherited members */
+  public static boolean AT_LINKS_NAVTREE = false;
   public static String outputPathBase = "/";
   public static ArrayList<String> inputPathHtmlDirs = new ArrayList<String>();
   public static ArrayList<String> inputPathHtmlDir2 = new ArrayList<String>();
+  public static String inputPathResourcesDir;
+  public static String outputPathResourcesDir;
   public static String outputPathHtmlDirs;
   public static String outputPathHtmlDir2;
-  public static final String devsiteRoot = "en/";
+  /* Javadoc output directory and included in url path */
   public static String javadocDir = "reference/";
   public static String htmlExtension;
 
@@ -87,10 +102,15 @@ public class Doclava {
   public static Map<Character, String> escapeChars = new HashMap<Character, String>();
   public static String title = "";
   public static SinceTagger sinceTagger = new SinceTagger();
+  public static ArtifactTagger artifactTagger = new ArtifactTagger();
   public static HashSet<String> knownTags = new HashSet<String>();
   public static FederationTagger federationTagger = new FederationTagger();
+  public static boolean showUnannotated = false;
   public static Set<String> showAnnotations = new HashSet<String>();
+  public static Set<String> hideAnnotations = new HashSet<String>();
+  public static boolean showAnnotationOverridesVisibility = false;
   public static Set<String> hiddenPackages = new HashSet<String>();
+  public static boolean includeAssets = true;
   public static boolean includeDefaultAssets = true;
   private static boolean generateDocs = true;
   private static boolean parseComments = false;
@@ -98,11 +118,21 @@ public class Doclava {
   public static boolean documentAnnotations = false;
   public static String documentAnnotationsPath = null;
   public static Map<String, String> annotationDocumentationMap = null;
+  public static boolean referenceOnly = false;
+  public static boolean staticOnly = false;
+  public static boolean yamlV2 = false; /* whether to build the new version of the yaml file */
+  public static AuxSource auxSource = new EmptyAuxSource();
+  public static Linter linter = new EmptyLinter();
+  public static boolean android = false;
+  public static String manifestFile = null;
+  public static Map<String, String> manifestPermissions = new HashMap<>();
 
   public static JSilver jSilver = null;
 
+  //API reference extensions
   private static boolean gmsRef = false;
   private static boolean gcmRef = false;
+  public static String libraryRoot = null;
   private static boolean samplesRef = false;
   private static boolean sac = false;
 
@@ -153,9 +183,16 @@ public class Doclava {
     // Create the dependency graph for the stubs  directory
     boolean offlineMode = false;
     String apiFile = null;
+    String dexApiFile = null;
     String removedApiFile = null;
+    String removedDexApiFile = null;
+    String exactApiFile = null;
+    String privateApiFile = null;
+    String privateDexApiFile = null;
     String debugStubsFile = "";
     HashSet<String> stubPackages = null;
+    HashSet<String> stubImportPackages = null;
+    boolean stubSourceOnly = false;
     ArrayList<String> knownTagsFiles = new ArrayList<String>();
 
     root = r;
@@ -170,6 +207,8 @@ public class Doclava {
         mHDFData.add(new String[] {a[1], a[2]});
       } else if (a[0].equals("-knowntags")) {
         knownTagsFiles.add(a[1]);
+      } else if (a[0].equals("-apidocsdir")) {
+        javadocDir = a[1];
       } else if (a[0].equals("-toroot")) {
         ClearPage.toroot = a[1];
       } else if (a[0].equals("-samplecode")) {
@@ -184,23 +223,33 @@ public class Doclava {
         ClearPage.htmlDirs = inputPathHtmlDirs;
       //the destination output path for additional htmldir
       } else if (a[0].equals("-htmldir2")) {
-          if (a[2].equals("default")) {
-          inputPathHtmlDirs.add(a[1]);
+        if (a[2].equals("default")) {
+          inputPathHtmlDir2.add(a[1]);
         } else {
           inputPathHtmlDir2.add(a[1]);
           outputPathHtmlDir2 = a[2];
         }
+      //the destination output path for additional resources (images)
+      } else if (a[0].equals("-resourcesdir")) {
+        inputPathResourcesDir = a[1];
+      } else if (a[0].equals("-resourcesoutdir")) {
+        outputPathResourcesDir = a[1];
       } else if (a[0].equals("-title")) {
         Doclava.title = a[1];
       } else if (a[0].equals("-werror")) {
         Errors.setWarningsAreErrors(true);
-      } else if (a[0].equals("-error") || a[0].equals("-warning") || a[0].equals("-hide")) {
+      } else if (a[0].equals("-lerror")) {
+        Errors.setLintsAreErrors(true);
+      } else if (a[0].equals("-error") || a[0].equals("-warning") || a[0].equals("-lint")
+          || a[0].equals("-hide")) {
         try {
           int level = -1;
           if (a[0].equals("-error")) {
             level = Errors.ERROR;
           } else if (a[0].equals("-warning")) {
             level = Errors.WARNING;
+          } else if (a[0].equals("-lint")) {
+            level = Errors.LINT;
           } else if (a[0].equals("-hide")) {
             level = Errors.HIDDEN;
           }
@@ -211,8 +260,14 @@ public class Doclava {
         }
       } else if (a[0].equals("-keeplist")) {
         keepListFile = a[1];
+      } else if (a[0].equals("-showUnannotated")) {
+        showUnannotated = true;
       } else if (a[0].equals("-showAnnotation")) {
         showAnnotations.add(a[1]);
+      } else if (a[0].equals("-hideAnnotation")) {
+        hideAnnotations.add(a[1]);
+      } else if (a[0].equals("-showAnnotationOverridesVisibility")) {
+        showAnnotationOverridesVisibility = true;
       } else if (a[0].equals("-hidePackage")) {
         hiddenPackages.add(a[1]);
       } else if (a[0].equals("-proguard")) {
@@ -238,25 +293,52 @@ public class Doclava {
         for (String pkg : a[1].split(":")) {
           stubPackages.add(pkg);
         }
+      } else if (a[0].equals("-stubimportpackages")) {
+        stubImportPackages = new HashSet<String>();
+        for (String pkg : a[1].split(":")) {
+          stubImportPackages.add(pkg);
+          hiddenPackages.add(pkg);
+        }
+      } else if (a[0].equals("-stubsourceonly")) {
+        stubSourceOnly = true;
       } else if (a[0].equals("-sdkvalues")) {
         sdkValuePath = a[1];
       } else if (a[0].equals("-api")) {
         apiFile = a[1];
+      } else if (a[0].equals("-dexApi")) {
+        dexApiFile = a[1];
       } else if (a[0].equals("-removedApi")) {
         removedApiFile = a[1];
-      }
-      else if (a[0].equals("-nodocs")) {
+      } else if (a[0].equals("-removedDexApi")) {
+        removedDexApiFile = a[1];
+      } else if (a[0].equals("-exactApi")) {
+        exactApiFile = a[1];
+      } else if (a[0].equals("-privateApi")) {
+        privateApiFile = a[1];
+      } else if (a[0].equals("-privateDexApi")) {
+        privateDexApiFile = a[1];
+      } else if (a[0].equals("-nodocs")) {
         generateDocs = false;
+      } else if (a[0].equals("-noassets")) {
+        includeAssets = false;
       } else if (a[0].equals("-nodefaultassets")) {
         includeDefaultAssets = false;
       } else if (a[0].equals("-parsecomments")) {
         parseComments = true;
       } else if (a[0].equals("-since")) {
         sinceTagger.addVersion(a[1], a[2]);
+      } else if (a[0].equals("-artifact")) {
+        artifactTagger.addArtifact(a[1], a[2]);
       } else if (a[0].equals("-offlinemode")) {
         offlineMode = true;
       } else if (a[0].equals("-metadataDebug")) {
         META_DBG = true;
+      } else if (a[0].equals("-includePreview")) {
+        INCLUDE_PREVIEW = true;
+      } else if (a[0].equals("-ignoreJdLinks")) {
+        if (DEVSITE_STATIC_ONLY) {
+          DEVSITE_IGNORE_JDLINKS = true;
+        }
       } else if (a[0].equals("-federate")) {
         try {
           String name = a[1];
@@ -272,19 +354,59 @@ public class Doclava {
         federationTagger.addSiteApi(name, file);
       } else if (a[0].equals("-yaml")) {
         yamlNavFile = a[1];
-      } else if (a[0].equals("-devsite")) {
-        // Don't copy the doclava assets to devsite output (ie use proj assets only)
-        includeDefaultAssets = false;
-        outputPathHtmlDirs = outputPathHtmlDirs + "/" + devsiteRoot;
+      } else if (a[0].equals("-dac_libraryroot")) {
+        libraryRoot = ensureSlash(a[1]);
+        mHDFData.add(new String[] {"library.root", a[1]});
+      } else if (a[0].equals("-dac_dataname")) {
+        mHDFData.add(new String[] {"dac_dataname", a[1]});
       } else if (a[0].equals("-documentannotations")) {
         documentAnnotations = true;
         documentAnnotationsPath = a[1];
+      } else if (a[0].equals("-referenceonly")) {
+        referenceOnly = true;
+        mHDFData.add(new String[] {"referenceonly", "1"});
+      } else if (a[0].equals("-staticonly")) {
+        staticOnly = true;
+        mHDFData.add(new String[] {"staticonly", "1"});
+      } else if (a[0].equals("-navtreeonly")) {
+        NAVTREE_ONLY = true;
+      } else if (a[0].equals("-atLinksNavtree")) {
+        AT_LINKS_NAVTREE = true;
+      } else if (a[0].equals("-yamlV2")) {
+        yamlV2 = true;
+      } else if (a[0].equals("-devsite")) {
+        // Don't copy any assets to devsite output
+        includeAssets = false;
+        USE_DEVSITE_LOCALE_OUTPUT_PATHS = true;
+        mHDFData.add(new String[] {"devsite", "1"});
+        if (staticOnly) {
+          DEVSITE_STATIC_ONLY = true;
+          System.out.println("  ... Generating static html only for devsite");
+        }
+        if (yamlNavFile == null) {
+          yamlNavFile = "_book.yaml";
+        }
+      } else if (a[0].equals("-android")) {
+        auxSource = new AndroidAuxSource();
+        linter = new AndroidLinter();
+        android = true;
+      } else if (a[0].equals("-manifest")) {
+        manifestFile = a[1];
       }
     }
 
+    // If the caller has not explicitly requested that unannotated classes and members should be
+    // shown in the output then only show them if no annotations were provided.
+    if (!showUnannotated && showAnnotations.isEmpty()) {
+      showUnannotated = true;
+    }
+
     if (!readKnownTagsFiles(knownTags, knownTagsFiles)) {
       return false;
     }
+    if (!readManifest()) {
+      return false;
+    }
 
     // Set up the data structures
     Converter.makeInfo(r);
@@ -298,6 +420,12 @@ public class Doclava {
       for (String tmpl : templates) {
         resourceLoaders.add(new FileSystemResourceLoader(tmpl));
       }
+      // If no custom template path is provided, and this is a devsite build,
+      // then use the bundled templates-sdk/ files by default
+      if (templates.isEmpty() && USE_DEVSITE_LOCALE_OUTPUT_PATHS) {
+        resourceLoaders.add(new ClassResourceLoader(Doclava.class, "/assets/templates-sdk"));
+        System.out.println("\n#########  OK, Using templates-sdk ############\n");
+      }
 
       templates = ClearPage.getBundledTemplateDirs();
       for (String tmpl : templates) {
@@ -313,92 +441,132 @@ public class Doclava {
         return false;
       }
 
-      //startTime = System.nanoTime();
-
-      // Apply @since tags from the XML file
-      sinceTagger.tagAll(Converter.rootClasses());
-
-      // Apply details of federated documentation
-      federationTagger.tagAll(Converter.rootClasses());
-
-      // Files for proofreading
-      if (proofreadFile != null) {
-        Proofread.initProofread(proofreadFile);
-      }
-      if (todoFile != null) {
-        TodoFile.writeTodoFile(todoFile);
-      }
-
-  if (samplesRef) {
-        // always write samples without offlineMode behaviors
-  writeSamples(false, sampleCodes, SORT_BY_NAV_GROUPS);
-  }
-
-      // HTML2 Pages -- Generate Pages from optional secondary dir
-      if (!inputPathHtmlDir2.isEmpty()) {
-        if (!outputPathHtmlDir2.isEmpty()) {
-          ClearPage.outputDir = outputPathBase + "/" + outputPathHtmlDir2;
+      // if requested, only generate the navtree for ds use-case
+      if (NAVTREE_ONLY) {
+        if (AT_LINKS_NAVTREE) {
+          AtLinksNavTree.writeAtLinksNavTree(javadocDir);
+        } else {
+          NavTree.writeNavTree(javadocDir, "");
         }
-        ClearPage.htmlDirs = inputPathHtmlDir2;
-        writeHTMLPages();
-        ClearPage.htmlDirs = inputPathHtmlDirs;
+        return true;
       }
 
-      // HTML Pages
-      if (!ClearPage.htmlDirs.isEmpty()) {
-        ClearPage.htmlDirs = inputPathHtmlDirs;
-        ClearPage.outputDir = outputPathHtmlDirs;
-        writeHTMLPages();
+      // don't do ref doc tasks in devsite static-only builds
+      if (!DEVSITE_STATIC_ONLY) {
+        // Load additional data structures from federated sites.
+        for(FederatedSite site : federationTagger.getSites()) {
+          Converter.addApiInfo(site.apiInfo());
+        }
+
+        // Apply @since tags from the XML file
+        sinceTagger.tagAll(Converter.rootClasses());
+
+        // Apply @artifact tags from the XML file
+        artifactTagger.tagAll(Converter.rootClasses());
+
+        // Apply details of federated documentation
+        federationTagger.tagAll(Converter.rootClasses());
+
+        // Files for proofreading
+        if (proofreadFile != null) {
+          Proofread.initProofread(proofreadFile);
+        }
+        if (todoFile != null) {
+          TodoFile.writeTodoFile(todoFile);
+        }
+
+        if (samplesRef) {
+          // always write samples without offlineMode behaviors
+          writeSamples(false, sampleCodes, SORT_BY_NAV_GROUPS);
+        }
       }
+      if (!referenceOnly) {
+        // HTML2 Pages -- Generate Pages from optional secondary dir
+        if (!inputPathHtmlDir2.isEmpty()) {
+          if (!outputPathHtmlDir2.isEmpty()) {
+            ClearPage.outputDir = outputPathBase + "/" + outputPathHtmlDir2;
+          }
+          ClearPage.htmlDirs = inputPathHtmlDir2;
+          writeHTMLPages();
+          ClearPage.htmlDirs = inputPathHtmlDirs;
+        }
+
+        // HTML Pages
+        if (!ClearPage.htmlDirs.isEmpty()) {
+          ClearPage.htmlDirs = inputPathHtmlDirs;
+          if (USE_DEVSITE_LOCALE_OUTPUT_PATHS) {
+            ClearPage.outputDir = outputPathHtmlDirs + "/en/";
+          } else {
+            ClearPage.outputDir = outputPathHtmlDirs;
+          }
+          writeHTMLPages();
+        }
+      }
+
+      writeResources();
 
       writeAssets();
 
-      // Navigation tree
-      String refPrefix = new String();
-      if(gmsRef){
-        refPrefix = "gms-";
-      } else if(gcmRef){
-        refPrefix = "gcm-";
+      // don't do ref doc tasks in devsite static-only builds
+      if (!DEVSITE_STATIC_ONLY) {
+        // Navigation tree
+        String refPrefix = new String();
+        if(gmsRef){
+          refPrefix = "gms-";
+        } else if(gcmRef){
+          refPrefix = "gcm-";
+        }
+        NavTree.writeNavTree(javadocDir, refPrefix);
+
+        // Write yaml tree.
+        if (yamlNavFile != null){
+          NavTree.writeYamlTree(javadocDir, yamlNavFile);
+          if (yamlV2) {
+            // Generate both for good measure, to make transitions easier, but change the filename
+            // for the new one so there's yet another explicit opt-in required by fixing the name.
+            yamlNavFile = "_NEW" + yamlNavFile;
+            NavTree.writeYamlTree2(javadocDir, yamlNavFile);
+          }
+        }
+
+        // Packages Pages
+        writePackages(refPrefix + "packages" + htmlExtension);
+
+        // Classes
+        writeClassLists();
+        writeClasses();
+        writeHierarchy();
+        // writeKeywords();
+
+        // Lists for JavaScript
+        writeLists();
+        if (keepListFile != null) {
+          writeKeepList(keepListFile);
+        }
+
+        Proofread.finishProofread(proofreadFile);
+
+        if (sdkValuePath != null) {
+          writeSdkValues(sdkValuePath);
+        }
       }
-      NavTree.writeNavTree(javadocDir, refPrefix);
-
-      // Write yaml tree.
-      if (yamlNavFile != null){
-        NavTree.writeYamlTree(javadocDir, yamlNavFile);
-      }
-
-      // Packages Pages
-      writePackages(javadocDir + refPrefix + "packages" + htmlExtension);
-
-      // Classes
-  writeClassLists();
-  writeClasses();
-  writeHierarchy();
-      // writeKeywords();
-
-      // Lists for JavaScript
-  writeLists();
-      if (keepListFile != null) {
-        writeKeepList(keepListFile);
-      }
-
-      // Index page
-  writeIndex();
-
-  Proofread.finishProofread(proofreadFile);
-
-  if (sdkValuePath != null) {
-    writeSdkValues(sdkValuePath);
-  }
-      // Write metadata for all processed files to jd_lists_unified.js in out dir
+      // Write metadata for all processed files to jd_lists_unified in out dir
       if (!sTaglist.isEmpty()) {
-        PageMetadata.WriteList(sTaglist);
+        PageMetadata.WriteListByLang(sTaglist);
+        // For devsite (ds) reference only, write samples_metadata to out dir
+        if ((USE_DEVSITE_LOCALE_OUTPUT_PATHS) && (!DEVSITE_STATIC_ONLY)) {
+          PageMetadata.WriteSamplesListByLang(sTaglist);
+        }
       }
     }
 
     // Stubs
-    if (stubsDir != null || apiFile != null || proguardFile != null || removedApiFile != null) {
-      Stubs.writeStubsAndApi(stubsDir, apiFile, proguardFile, removedApiFile, stubPackages);
+    if (stubsDir != null || apiFile != null || dexApiFile != null || proguardFile != null
+        || removedApiFile != null || removedDexApiFile != null || exactApiFile != null
+        || privateApiFile != null || privateDexApiFile != null) {
+      Stubs.writeStubsAndApi(stubsDir, apiFile, dexApiFile, proguardFile, removedApiFile,
+          removedDexApiFile, exactApiFile, privateApiFile, privateDexApiFile, stubPackages,
+          stubImportPackages, stubSourceOnly);
     }
 
     Errors.printErrors();
@@ -410,9 +578,9 @@ public class Doclava {
     return !Errors.hadError;
   }
 
-  private static void writeIndex() {
+  private static void writeIndex(String dir) {
     Data data = makeHDF();
-    ClearPage.write(data, "index.cs", javadocDir + "index" + htmlExtension);
+    ClearPage.write(data, "index.cs", dir + "index" + htmlExtension);
   }
 
   private static boolean readTemplateSettings() {
@@ -488,6 +656,32 @@ public class Doclava {
         return true;
     }
 
+  private static boolean readManifest() {
+    manifestPermissions.clear();
+    if (manifestFile == null) {
+      return true;
+    }
+    try (BufferedInputStream in = new BufferedInputStream(new FileInputStream(manifestFile));
+        ByteArrayOutputStream out = new ByteArrayOutputStream()) {
+      byte[] buffer = new byte[1024];
+      int count;
+      while ((count = in.read(buffer)) != -1) {
+        out.write(buffer, 0, count);
+      }
+      final Matcher m = Pattern.compile("(?s)<permission "
+          + "[^>]*android:name=\"([^\">]+)\""
+          + "[^>]*android:protectionLevel=\"([^\">]+)\"").matcher(out.toString());
+      while (m.find()) {
+        manifestPermissions.put(m.group(1), m.group(2));
+      }
+    } catch (IOException e) {
+      Errors.error(Errors.PARSE_ERROR, (SourcePositionInfo) null,
+          "Failed to parse " + manifestFile + ": " + e);
+      return false;
+    }
+    return true;
+  }
+
   public static String escape(String s) {
     if (escapeChars.size() == 0) {
       return s;
@@ -545,6 +739,9 @@ public class Doclava {
     if (option.equals("-knowntags")) {
       return 2;
     }
+    if (option.equals("-apidocsdir")) {
+      return 2;
+    }
     if (option.equals("-toroot")) {
       return 2;
     }
@@ -562,18 +759,39 @@ public class Doclava {
     if (option.equals("-devsite")) {
       return 1;
     }
+    if (option.equals("-yamlV2")) {
+      return 1;
+    }
+    if (option.equals("-dac_libraryroot")) {
+      return 2;
+    }
+    if (option.equals("-dac_dataname")) {
+      return 2;
+    }
+    if (option.equals("-ignoreJdLinks")) {
+      return 1;
+    }
     if (option.equals("-htmldir")) {
       return 2;
     }
     if (option.equals("-htmldir2")) {
       return 3;
     }
+    if (option.equals("-resourcesdir")) {
+      return 2;
+    }
+    if (option.equals("-resourcesoutdir")) {
+      return 2;
+    }
     if (option.equals("-title")) {
       return 2;
     }
     if (option.equals("-werror")) {
       return 1;
     }
+    if (option.equals("-lerror")) {
+      return 1;
+    }
     if (option.equals("-hide")) {
       return 2;
     }
@@ -586,9 +804,18 @@ public class Doclava {
     if (option.equals("-keeplist")) {
       return 2;
     }
+    if (option.equals("-showUnannotated")) {
+      return 1;
+    }
     if (option.equals("-showAnnotation")) {
       return 2;
     }
+    if (option.equals("-hideAnnotation")) {
+      return 2;
+    }
+    if (option.equals("-showAnnotationOverridesVisibility")) {
+      return 1;
+    }
     if (option.equals("-hidePackage")) {
       return 2;
     }
@@ -622,15 +849,36 @@ public class Doclava {
     if (option.equals("-stubpackages")) {
       return 2;
     }
+    if (option.equals("-stubimportpackages")) {
+      return 2;
+    }
+    if (option.equals("-stubsourceonly")) {
+      return 1;
+    }
     if (option.equals("-sdkvalues")) {
       return 2;
     }
     if (option.equals("-api")) {
       return 2;
     }
+    if (option.equals("-dexApi")) {
+      return 2;
+    }
     if (option.equals("-removedApi")) {
       return 2;
     }
+    if (option.equals("-removedDexApi")) {
+      return 2;
+    }
+    if (option.equals("-exactApi")) {
+      return 2;
+    }
+    if (option.equals("-privateApi")) {
+      return 2;
+    }
+    if (option.equals("-privateDexApi")) {
+      return 2;
+    }
     if (option.equals("-nodocs")) {
       return 1;
     }
@@ -643,6 +891,9 @@ public class Doclava {
     if (option.equals("-since")) {
       return 3;
     }
+    if (option.equals("-artifact")) {
+      return 3;
+    }
     if (option.equals("-offlinemode")) {
       return 1;
     }
@@ -655,9 +906,6 @@ public class Doclava {
     if (option.equals("-yaml")) {
       return 2;
     }
-    if (option.equals("-devsite")) {
-      return 1;
-    }
     if (option.equals("-gmsref")) {
       gmsRef = true;
       return 1;
@@ -669,9 +917,30 @@ public class Doclava {
     if (option.equals("-metadataDebug")) {
       return 1;
     }
+    if (option.equals("-includePreview")) {
+      return 1;
+    }
     if (option.equals("-documentannotations")) {
       return 2;
     }
+    if (option.equals("-referenceonly")) {
+      return 1;
+    }
+    if (option.equals("-staticonly")) {
+      return 1;
+    }
+    if (option.equals("-navtreeonly")) {
+      return 1;
+    }
+    if (option.equals("-atLinksNavtree")) {
+      return 1;
+    }
+    if (option.equals("-android")) {
+      return 1;
+    }
+    if (option.equals("-manifest")) {
+      return 2;
+    }
     return 0;
   }
   public static boolean validOptions(String[][] options, DocErrorReporter r) {
@@ -699,11 +968,9 @@ public class Doclava {
     return data;
   }
 
-
-
   public static Data makePackageHDF() {
     Data data = makeHDF();
-    ClassInfo[] classes = Converter.rootClasses();
+    Collection<ClassInfo> classes = Converter.rootClasses();
 
     SortedMap<String, PackageInfo> sorted = new TreeMap<String, PackageInfo>();
     for (ClassInfo cl : classes) {
@@ -773,6 +1040,7 @@ public class Doclava {
       }
       data.setValue("reference", "1");
       data.setValue("reference.apilevels", sinceTagger.hasVersions() ? "1" : "0");
+      data.setValue("reference.artifacts", artifactTagger.hasArtifacts() ? "1" : "0");
       data.setValue("docs.packages." + i + ".name", s);
       data.setValue("docs.packages." + i + ".link", pkg.htmlPage());
       data.setValue("docs.packages." + i + ".since", pkg.getSince());
@@ -793,18 +1061,19 @@ public class Doclava {
         String templ = relative + f.getName();
         int len = templ.length();
         if (len > 3 && ".cs".equals(templ.substring(len - 3))) {
-          Data data = makeHDF();
+          Data data = makePackageHDF();
           String filename = templ.substring(0, len - 3) + htmlExtension;
           ClearPage.write(data, templ, filename, js);
         } else if (len > 3 && ".jd".equals(templ.substring(len - 3))) {
-          Data data = makeHDF();
+          Data data = makePackageHDF();
           String filename = templ.substring(0, len - 3) + htmlExtension;
           DocFile.writePage(f.getAbsolutePath(), relative, filename, data);
         } else if(!f.getName().equals(".DS_Store")){
               Data data = makeHDF();
               String hdfValue = data.getValue("sac") == null ? "" : data.getValue("sac");
               boolean allowExcepted = hdfValue.equals("true") ? true : false;
-              ClearPage.copyFile(allowExcepted, f, templ);
+              boolean append = false;
+              ClearPage.copyFile(allowExcepted, f, templ, append);
         }
       } else if (f.isDirectory()) {
         writeDirectory(f, relative + f.getName() + "/", js);
@@ -826,7 +1095,27 @@ public class Doclava {
     }
   }
 
+  /* copy files supplied by the -resourcesdir flag */
+  public static void writeResources() {
+    if (inputPathResourcesDir != null && !inputPathResourcesDir.isEmpty()) {
+      try {
+        File f = new File(inputPathResourcesDir);
+        if (!f.isDirectory()) {
+          System.err.println("resourcesdir is not a directory: " + inputPathResourcesDir);
+          return;
+        }
+
+        ResourceLoader loader = new FileSystemResourceLoader(f);
+        JSilver js = new JSilver(loader);
+        writeDirectory(f, outputPathResourcesDir, js);
+      } catch(Exception e) {
+        System.err.println("Could not copy resourcesdir: " + e);
+      }
+    }
+  }
+
   public static void writeAssets() {
+    if (!includeAssets) return;
     JarFile thisJar = JarUtils.jarForClass(Doclava.class, null);
     if ((thisJar != null) && (includeDefaultAssets)) {
       try {
@@ -863,7 +1152,7 @@ public class Doclava {
     // Write the lists for API references
     Data data = makeHDF();
 
-    ClassInfo[] classes = Converter.rootClasses();
+    Collection<ClassInfo> classes = Converter.rootClasses();
 
     SortedMap<String, Object> sorted = new TreeMap<String, Object>();
     for (ClassInfo cl : classes) {
@@ -882,6 +1171,12 @@ public class Doclava {
     }
 
     int i = 0;
+    String listDir = javadocDir;
+    if (USE_DEVSITE_LOCALE_OUTPUT_PATHS) {
+      if (libraryRoot != null) {
+        listDir = listDir + libraryRoot;
+      }
+    }
     for (String s : sorted.keySet()) {
       data.setValue("docs.pages." + i + ".id", "" + i);
       data.setValue("docs.pages." + i + ".label", s);
@@ -900,11 +1195,12 @@ public class Doclava {
       }
       i++;
     }
-    ClearPage.write(data, "lists.cs", javadocDir + "lists.js");
+    ClearPage.write(data, "lists.cs", listDir + "lists.js");
 
 
     // Write the lists for JD documents (if there are HTML directories to process)
-    if (inputPathHtmlDirs.size() > 0) {
+    // Skip this for devsite builds
+    if ((inputPathHtmlDirs.size() > 0) && (!USE_DEVSITE_LOCALE_OUTPUT_PATHS)) {
       Data jddata = makeHDF();
       Iterator counter = new Iterator();
       for (String htmlDir : inputPathHtmlDirs) {
@@ -1044,8 +1340,8 @@ public class Doclava {
    */
   public static void writeKeepList(String filename) {
     HashSet<ClassInfo> notStrippable = new HashSet<ClassInfo>();
-    ClassInfo[] all = Converter.allClasses();
-    Arrays.sort(all); // just to make the file a little more readable
+    Collection<ClassInfo> all = Converter.allClasses().stream().sorted(ClassInfo.comparator)
+        .collect(Collectors.toList());
 
     // If a class is public and not hidden, then it and everything it derives
     // from cannot be stripped. Otherwise we can strip it.
@@ -1076,7 +1372,7 @@ public class Doclava {
       return sVisiblePackages;
     }
 
-    ClassInfo[] classes = Converter.rootClasses();
+    Collection<ClassInfo> classes = Converter.rootClasses();
     SortedMap<String, PackageInfo> sorted = new TreeMap<String, PackageInfo>();
     for (ClassInfo cl : classes) {
       PackageInfo pkg = cl.containingPackage();
@@ -1165,8 +1461,15 @@ public class Doclava {
 
     TagInfo.makeHDF(data, "root.descr", Converter.convertTags(root.inlineTags(), null));
 
-    ClearPage.write(data, "packages.cs", filename);
-    ClearPage.write(data, "package-list.cs", javadocDir + "package-list");
+    String packageDir = javadocDir;
+    if (USE_DEVSITE_LOCALE_OUTPUT_PATHS) {
+      if (libraryRoot != null) {
+        packageDir = packageDir + libraryRoot;
+      }
+    }
+    data.setValue("page.not-api", "true");
+    ClearPage.write(data, "packages.cs", packageDir + filename);
+    ClearPage.write(data, "package-list.cs", packageDir + "package-list");
 
     Proofread.writePackages(filename, Converter.convertTags(root.inlineTags(), null));
   }
@@ -1248,8 +1551,19 @@ public class Doclava {
       cl.makeShortDescrHDF(data, "docs.classes." + first + '.' + i);
     }
 
+    String packageDir = javadocDir;
+    if (USE_DEVSITE_LOCALE_OUTPUT_PATHS) {
+      if (libraryRoot != null) {
+        packageDir = packageDir + libraryRoot;
+      }
+    }
+
+    data.setValue("page.not-api", "true");
     setPageTitle(data, "Class Index");
-    ClearPage.write(data, "classes.cs", javadocDir + "classes" + htmlExtension);
+    ClearPage.write(data, "classes.cs", packageDir + "classes" + htmlExtension);
+
+    // Index page redirects to the classes.html page, so use the same directory
+    writeIndex(packageDir);
   }
 
   // we use the word keywords because "index" means something else in html land
@@ -1274,7 +1588,7 @@ public class Doclava {
    */
 
   public static void writeHierarchy() {
-    ClassInfo[] classes = Converter.rootClasses();
+    Collection<ClassInfo> classes = Converter.rootClasses();
     ArrayList<ClassInfo> info = new ArrayList<ClassInfo>();
     for (ClassInfo cl : classes) {
       if (!cl.isHiddenOrRemoved()) {
@@ -1288,7 +1602,7 @@ public class Doclava {
   }
 
   public static void writeClasses() {
-    ClassInfo[] classes = Converter.rootClasses();
+    Collection<ClassInfo> classes = Converter.rootClasses();
 
     for (ClassInfo cl : classes) {
       Data data = makePackageHDF();
@@ -1488,7 +1802,7 @@ public class Doclava {
     ArrayList<ClassInfo> widgets = new ArrayList<ClassInfo>();
     ArrayList<ClassInfo> layoutParams = new ArrayList<ClassInfo>();
 
-    ClassInfo[] classes = Converter.allClasses();
+    Collection<ClassInfo> classes = Converter.allClasses();
 
     // The topmost LayoutParams class - android.view.ViewGroup.LayoutParams
     ClassInfo topLayoutParams = null;
