@@ -29,6 +29,7 @@ import org.antlr.runtime.tree.Tree;
 
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -1069,7 +1070,7 @@ public class InfoBuilder {
                 modifiers.isPrivate(), modifiers.isFinal(),
                 modifiers.isStatic(), modifiers.isSynthetic(),
                 modifiers.isAbstract(), modifiers.isSynchronized(),
-                false, isAnnotationElement, kind, flatSignature.toString(),
+                false, modifiers.isDefault(), isAnnotationElement, kind, flatSignature.toString(),
                 null, returnType, parameters, thrownExceptions,
                 commentAndPosition.getPosition(), modifiers.getAnnotations());
 
@@ -1142,7 +1143,8 @@ public class InfoBuilder {
                     commentAndPosition.setPosition(paramPart);
 
                     parameters.add(new ParameterInfo(name, type.qualifiedTypeName(), type,
-                            isVarArg, commentAndPosition.getPosition()));
+                            isVarArg, commentAndPosition.getPosition(),
+                            Collections.<AnnotationInstanceInfo>emptyList()));
                 }
             }
         }
@@ -1852,6 +1854,7 @@ public class InfoBuilder {
         private boolean mIsSynthetic = false;
         private boolean mIsSynchronized = false;
         private boolean mIsStrictfp = false;
+        private boolean mIsDefault = false;
         private InfoBuilder mBuilder;
         private ArrayList<AnnotationInstanceInfo> mAnnotations;
 
@@ -1893,6 +1896,8 @@ public class InfoBuilder {
                     mIsSynchronized = true;
                 }  else if ("strictfp".equals(modifier)) {
                     mIsStrictfp = true;
+                }  else if ("default".equals(modifier)) {
+                    mIsDefault = true;
                 } else if ("annotation".equals(modifier)) {
                     mAnnotations.add(buildAnnotationInstance((ParseTree) child, mBuilder));
                 }
@@ -1948,6 +1953,10 @@ public class InfoBuilder {
             return mIsStrictfp;
         }
 
+        public boolean isDefault() {
+            return mIsDefault;
+        }
+
         public ArrayList<AnnotationInstanceInfo> getAnnotations() {
             return mAnnotations;
         }
