@@ -20,12 +20,23 @@ import com.google.clearsilver.jsilver.data.Data;
 import com.google.doclava.apicheck.AbstractMethodInfo;
 import com.google.doclava.apicheck.ApiInfo;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Comparator;
+import java.util.HashSet;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.Objects;
+import java.util.Queue;
+import java.util.function.Predicate;
 
 public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolvable {
   public static final Comparator<MethodInfo> comparator = new Comparator<MethodInfo>() {
+    @Override
     public int compare(MethodInfo a, MethodInfo b) {
-        return a.name().compareTo(b.name());
+      // TODO: expand to compare signature for better sorting
+      return a.name().compareTo(b.name());
     }
   };
 
@@ -101,7 +112,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     ArrayList<ClassInfo> queue = new ArrayList<ClassInfo>();
     if (containingClass().realSuperclass() != null
         && containingClass().realSuperclass().isAbstract()) {
-      queue.add(containingClass());
+      queue.add(containingClass().realSuperclass());
     }
     addInterfaces(containingClass().realInterfaces(), queue);
     for (ClassInfo iface : queue) {
@@ -116,32 +127,22 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     return null;
   }
 
-  public MethodInfo findSuperclassImplementation(HashSet notStrippable) {
+  public MethodInfo findPredicateOverriddenMethod(Predicate<MemberInfo> predicate) {
     if (mReturnType == null) {
       // ctor
       return null;
     }
     if (mOverriddenMethod != null) {
-      // Even if we're told outright that this was the overridden method, we want to
-      // be conservative and ignore mismatches of parameter types -- they arise from
-      // extending generic specializations, and we want to consider the derived-class
-      // method to be a non-override.
-      if (this.signature().equals(mOverriddenMethod.signature())) {
+      if (equals(mOverriddenMethod) && !mOverriddenMethod.isStatic()
+          && predicate.test(mOverriddenMethod)) {
         return mOverriddenMethod;
       }
     }
 
-    ArrayList<ClassInfo> queue = new ArrayList<ClassInfo>();
-    if (containingClass().realSuperclass() != null
-        && containingClass().realSuperclass().isAbstract()) {
-      queue.add(containingClass());
-    }
-    addInterfaces(containingClass().realInterfaces(), queue);
-    for (ClassInfo iface : queue) {
-      for (MethodInfo me : iface.methods()) {
-        if (me.name().equals(this.name()) && me.signature().equals(this.signature())
-            && notStrippable.contains(me.containingClass())) {
-          return me;
+    for (ClassInfo clazz : containingClass().gatherAncestorClasses()) {
+      for (MethodInfo method : clazz.getExhaustiveMethods()) {
+        if (equals(method) && !method.isStatic() && predicate.test(method)) {
+          return method;
         }
       }
     }
@@ -160,7 +161,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     ArrayList<ClassInfo> queue = new ArrayList<ClassInfo>();
     if (containingClass().realSuperclass() != null
         && containingClass().realSuperclass().isAbstract()) {
-      queue.add(containingClass());
+      queue.add(containingClass().realSuperclass());
     }
     addInterfaces(containingClass().realInterfaces(), queue);
     for (ClassInfo iface : queue) {
@@ -216,10 +217,18 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
         }
       }
 
-      if (commentDeprecated != annotationDeprecated) {
+      // Check to see that the JavaDoc contains @deprecated AND the method is marked as @Deprecated.
+      // Otherwise, warn.
+      // Note: We only do this for "included" classes (i.e. those we have source code for); we do
+      // not have comments for classes from .class files but we do know whether a method is marked
+      // as @Deprecated.
+      if (mContainingClass.isIncluded() && !isHiddenOrRemoved()
+          && commentDeprecated != annotationDeprecated) {
         Errors.error(Errors.DEPRECATION_MISMATCH, position(), "Method "
             + mContainingClass.qualifiedName() + "." + name()
-            + ": @Deprecated annotation and @deprecated doc tag do not match");
+            + ": @Deprecated annotation (" + (annotationDeprecated ? "" : "not ")
+            + "present) and @deprecated doc tag (" + (commentDeprecated ? "" : "not ")
+            + "present) do not match");
       }
 
       mIsDeprecated = commentDeprecated | annotationDeprecated;
@@ -227,7 +236,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     }
     return mIsDeprecated;
   }
-  
+
   public void setDeprecated(boolean deprecated) {
     mDeprecatedKnown = true;
     mIsDeprecated = deprecated;
@@ -237,13 +246,22 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     return mTypeParameters;
   }
 
+  public boolean hasTypeParameters() {
+      return mTypeParameters != null && !mTypeParameters.isEmpty();
+  }
+
   /**
    * Clone this MethodInfo as if it belonged to the specified ClassInfo and apply the
    * typeArgumentMapping to the parameters and return types.
    */
   public MethodInfo cloneForClass(ClassInfo newContainingClass,
       Map<String, TypeInfo> typeArgumentMapping) {
-    TypeInfo returnType = mReturnType.getTypeWithArguments(typeArgumentMapping);
+    if (newContainingClass == containingClass()) {
+      return this;
+    }
+    TypeInfo returnType = (mReturnType != null)
+        ? mReturnType.getTypeWithArguments(typeArgumentMapping)
+        : null;
     ArrayList<ParameterInfo> parameters = new ArrayList<ParameterInfo>();
     for (ParameterInfo pi : mParameters) {
       parameters.add(pi.cloneWithTypeArguments(typeArgumentMapping));
@@ -252,7 +270,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
         new MethodInfo(getRawCommentText(), mTypeParameters, name(), signature(),
             newContainingClass, realContainingClass(), isPublic(), isProtected(),
             isPackagePrivate(), isPrivate(), isFinal(), isStatic(), isSynthetic(), mIsAbstract,
-            mIsSynchronized, mIsNative, mIsAnnotationElement, kind(), mFlatSignature,
+            mIsSynchronized, mIsNative, mIsDefault, mIsAnnotationElement, kind(), mFlatSignature,
             mOverriddenMethod, returnType, mParameters, mThrownExceptions, position(),
             annotations());
     result.init(mDefaultAnnotationElementValue);
@@ -263,10 +281,10 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
       String signature, ClassInfo containingClass, ClassInfo realContainingClass, boolean isPublic,
       boolean isProtected, boolean isPackagePrivate, boolean isPrivate, boolean isFinal,
       boolean isStatic, boolean isSynthetic, boolean isAbstract, boolean isSynchronized,
-      boolean isNative, boolean isAnnotationElement, String kind, String flatSignature,
-      MethodInfo overriddenMethod, TypeInfo returnType, ArrayList<ParameterInfo> parameters,
-      ArrayList<ClassInfo> thrownExceptions, SourcePositionInfo position,
-      ArrayList<AnnotationInstanceInfo> annotations) {
+      boolean isNative, boolean isDefault, boolean isAnnotationElement, String kind,
+      String flatSignature, MethodInfo overriddenMethod, TypeInfo returnType,
+      ArrayList<ParameterInfo> parameters, ArrayList<ClassInfo> thrownExceptions,
+      SourcePositionInfo position, ArrayList<AnnotationInstanceInfo> annotations) {
     // Explicitly coerce 'final' state of Java6-compiled enum values() method, to match
     // the Java5-emitted base API description.
     super(rawCommentText, name, signature, containingClass, realContainingClass, isPublic,
@@ -274,19 +292,13 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
         ((name.equals("values") && containingClass.isEnum()) ? true : isFinal),
         isStatic, isSynthetic, kind, position, annotations);
 
-    // The underlying MethodDoc for an interface's declared methods winds up being marked
-    // non-abstract. Correct that here by looking at the immediate-parent class, and marking
-    // this method abstract if it is an unimplemented interface method.
-    if (containingClass.isInterface()) {
-      isAbstract = true;
-    }
-
     mReasonOpened = "0:0";
     mIsAnnotationElement = isAnnotationElement;
     mTypeParameters = typeParameters;
     mIsAbstract = isAbstract;
     mIsSynchronized = isSynchronized;
     mIsNative = isNative;
+    mIsDefault = isDefault;
     mFlatSignature = flatSignature;
     mOverriddenMethod = overriddenMethod;
     mReturnType = returnType;
@@ -310,6 +322,10 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     return mIsNative;
   }
 
+  public boolean isDefault() {
+    return mIsDefault;
+  }
+
   public String flatSignature() {
     return mFlatSignature;
   }
@@ -318,6 +334,10 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     return new InlineTags();
   }
 
+  public TagInfo[] blockTags() {
+    return comment().blockTags();
+  }
+
   public InheritedTags firstSentenceTags() {
     return new FirstSentenceTags();
   }
@@ -329,7 +349,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
   public TypeInfo returnType() {
     return mReturnType;
   }
-  
+
   public String prettySignature() {
     return name() + prettyParameters();
   }
@@ -337,7 +357,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
   public String prettyQualifiedSignature() {
     return qualifiedName() + prettyParameters();
   }
-  
+
   /**
    * Returns a printable version of the parameters of this method's signature.
    */
@@ -349,7 +369,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
       }
       params.append(pInfo.type().simpleTypeName());
     }
-    
+
     params.append(")");
     return params.toString();
   }
@@ -415,79 +435,102 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     return mThrowsTags;
   }
 
-  private static int indexOfParam(String name, String[] list) {
+  private static int indexOfParam(String name, ParamTagInfo[] list) {
     final int N = list.length;
     for (int i = 0; i < N; i++) {
-      if (name.equals(list[i])) {
+      if (name.equals(list[i].parameterName())) {
         return i;
       }
     }
     return -1;
   }
 
+  /* Checks whether the name documented with the provided @param tag
+   * actually matches one of the method parameters. */
+  private boolean isParamTagInMethod(ParamTagInfo tag) {
+    for (ParameterInfo paramInfo : mParameters) {
+      if (paramInfo.name().equals(tag.parameterName())) {
+        return true;
+      }
+    }
+    return false;
+  }
+
   public ParamTagInfo[] paramTags() {
     if (mParamTags == null) {
       final int N = mParameters.size();
+      final String DEFAULT_COMMENT = "<!-- no parameter comment -->";
 
       if (N == 0) {
           // Early out for empty case.
           mParamTags = ParamTagInfo.EMPTY_ARRAY;
           return ParamTagInfo.EMPTY_ARRAY;
       }
+      // Where we put each result
+      mParamTags = ParamTagInfo.getArray(N);
 
-      String[] names = new String[N];
-      String[] comments = new String[N];
-      SourcePositionInfo[] positions = new SourcePositionInfo[N];
+      // collect all the @param tag info
+      ParamTagInfo[] paramTags = comment().paramTags();
 
-      // get the right names so we can handle our names being different from
-      // our parent's names.
+      // Complain about misnamed @param tags
+      for (ParamTagInfo tag : paramTags) {
+        if (!isParamTagInMethod(tag) && !tag.isTypeParameter()){
+          Errors.error(Errors.UNKNOWN_PARAM_TAG_NAME, tag.position(),
+              "@param tag with name that doesn't match the parameter list: '"
+              + tag.parameterName() + "'");
+        }
+      }
+
+      // Loop the parameters known from the method signature...
+      // Start by getting the known parameter name and data type. Then, if
+      // there's an @param tag that matches the current parameter name, get the
+      // javadoc comments. But if there's no @param comments here, then
+      // check if it's available from the parent class.
       int i = 0;
       for (ParameterInfo param : mParameters) {
-        names[i] = param.name();
-        comments[i] = "";
-        positions[i] = param.position();
-        i++;
-      }
+        String name = param.name();
+        String type = param.type().simpleTypeName();
+        String comment = DEFAULT_COMMENT;
+        SourcePositionInfo position = param.position();
 
-      // gather our comments, and complain about misnamed @param tags
-      for (ParamTagInfo tag : comment().paramTags()) {
-        int index = indexOfParam(tag.parameterName(), names);
+        // Find the matching param from the @param tags in order to get
+        // the parameter comments
+        int index = indexOfParam(name, paramTags);
         if (index >= 0) {
-          comments[index] = tag.parameterComment();
-          positions[index] = tag.position();
-        } else {
-          Errors.error(Errors.UNKNOWN_PARAM_TAG_NAME, tag.position(),
-              "@param tag with name that doesn't match the parameter list: '" + tag.parameterName()
-                  + "'");
+          comment = paramTags[index].parameterComment();
+          position = paramTags[index].position();
         }
-      }
 
-      // get our parent's tags to fill in the blanks
-      MethodInfo overridden = this.findOverriddenMethod(name(), signature());
-      if (overridden != null) {
-        ParamTagInfo[] maternal = overridden.paramTags();
-        for (i = 0; i < N; i++) {
-          if (comments[i].equals("")) {
-            comments[i] = maternal[i].parameterComment();
-            positions[i] = maternal[i].position();
+        // get our parent's tags to fill in the blanks
+        MethodInfo overridden = this.findOverriddenMethod(name(), signature());
+        if (overridden != null) {
+          ParamTagInfo[] maternal = overridden.paramTags();
+          if (comment.equals(DEFAULT_COMMENT)) {
+            comment = maternal[i].parameterComment();
+            position = maternal[i].position();
           }
         }
-      }
 
-      // construct the results, and cache them for next time
-      mParamTags = ParamTagInfo.getArray(N);
-      for (i = 0; i < N; i++) {
+        // Collect all docs requested by annotations
+        TagInfo[] auxTags = Doclava.auxSource.paramAuxTags(this, param, comment);
+
+        // Okay, now add the collected parameter information to the method data
         mParamTags[i] =
-            new ParamTagInfo("@param", "@param", names[i] + " " + comments[i], parent(),
-                positions[i]);
+            new ParamTagInfo("@param", type, name + " " + comment, parent(),
+                position, auxTags);
 
-        // while we're here, if we find any parameters that are still undocumented at this
-        // point, complain. (this warning is off by default, because it's really, really
-        // common; but, it's good to be able to enforce it)
-        if (comments[i].equals("")) {
-          Errors.error(Errors.UNDOCUMENTED_PARAMETER, positions[i], "Undocumented parameter '"
-              + names[i] + "' on method '" + name() + "'");
+        // while we're here, if we find any parameters that are still
+        // undocumented at this point, complain. This warning is off by
+        // default, because it's really, really common;
+        // but, it's good to be able to enforce it.
+        if (comment.equals(DEFAULT_COMMENT)) {
+          Errors.error(Errors.UNDOCUMENTED_PARAMETER, position,
+              "Undocumented parameter '" + name + "' on method '"
+              + name() + "'");
+        } else {
+          Doclava.linter.lintParameter(this, param, position, mParamTags[i]);
         }
+        i++;
       }
     }
     return mParamTags;
@@ -527,16 +570,16 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
       }
       int i = 0;
       for (ParameterInfo mine : mParameters) {
-        if (!mine.matchesDimension(dimensions[i], varargs)) {
+        // If the method we're matching against is a varargs method (varargs == true), then
+        // only its last parameter is varargs.
+        if (!mine.matchesDimension(dimensions[i], (i == params.length - 1) ? varargs : false)) {
           return false;
         }
         TypeInfo myType = mine.type();
         String qualifiedName = myType.qualifiedTypeName();
         String realType = myType.isPrimitive() ? "" : myType.asClassInfo().qualifiedName();
         String s = params[i];
-        int slen = s.length();
-        int qnlen = qualifiedName.length();
-        
+
         // Check for a matching generic name or best known type
         if (!matchesType(qualifiedName, s) && !matchesType(realType, s)) {
           return false;
@@ -546,7 +589,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     }
     return true;
   }
-  
+
   /**
    * Checks to see if a parameter from a method signature is
    * compatible with a parameter given in a {@code @link} tag.
@@ -575,12 +618,15 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
       data.setValue(base + ".abstract", mIsAbstract ? "abstract" : "");
     }
 
+    data.setValue(base + ".default", mIsDefault ? "default" : "");
     data.setValue(base + ".synchronized", mIsSynchronized ? "synchronized" : "");
     data.setValue(base + ".final", isFinal() ? "final" : "");
     data.setValue(base + ".static", isStatic() ? "static" : "");
 
     TagInfo.makeHDF(data, base + ".shortDescr", firstSentenceTags());
     TagInfo.makeHDF(data, base + ".descr", inlineTags());
+    TagInfo.makeHDF(data, base + ".descrAux", Doclava.auxSource.methodAuxTags(this));
+    TagInfo.makeHDF(data, base + ".blockTags", blockTags());
     TagInfo.makeHDF(data, base + ".deprecated", deprecatedTags());
     TagInfo.makeHDF(data, base + ".seeAlso", seeTags());
     data.setValue(base + ".since", getSince());
@@ -598,6 +644,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
       data.setValue(base + ".scope", "public");
     }
     TagInfo.makeHDF(data, base + ".returns", returnTags());
+    TagInfo.makeHDF(data, base + ".returnsAux", Doclava.auxSource.returnAuxTags(this));
 
     if (mTypeParameters != null) {
       TypeInfo.makeHDF(data, base + ".generic.typeArguments", mTypeParameters, false);
@@ -614,13 +661,14 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
       }
     }
 
-
     AnnotationInstanceInfo.makeLinkListHDF(
       data,
       base + ".showAnnotations",
       showAnnotations().toArray(new AnnotationInstanceInfo[showAnnotations().size()]));
 
     setFederatedReferences(data, base);
+
+    Doclava.linter.lintMethod(this);
   }
 
   public HashSet<String> typeVariables() {
@@ -646,7 +694,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
   }
 
   public String typeArgumentsName(HashSet<String> typeVars) {
-    if (mTypeParameters == null || mTypeParameters.isEmpty()) {
+    if (!hasTypeParameters()) {
       return "";
     } else {
       return TypeInfo.typeArgumentsName(mTypeParameters, typeVars);
@@ -685,6 +733,23 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     return this.name();
   }
 
+  @Override
+  public boolean equals(Object o) {
+    if (this == o) {
+      return true;
+    } else if (o instanceof MethodInfo) {
+      final MethodInfo m = (MethodInfo) o;
+      return mName.equals(m.mName) && signature().equals(m.signature());
+    } else {
+      return false;
+    }
+  }
+
+  @Override
+  public int hashCode() {
+    return Objects.hash(mName, signature());
+  }
+
   public void setReason(String reason) {
     mReasonOpened = reason;
   }
@@ -692,13 +757,13 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
   public String getReason() {
     return mReasonOpened;
   }
-  
+
   public void addException(String exec) {
     ClassInfo exceptionClass = new ClassInfo(exec);
 
     mThrownExceptions.add(exceptionClass);
   }
-  
+
   public void addParameter(ParameterInfo p) {
     // Name information
     if (mParameters == null) {
@@ -718,6 +783,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
   private boolean mIsVarargs;
   private boolean mDeprecatedKnown;
   private boolean mIsDeprecated;
+  private boolean mIsDefault;
   private ArrayList<ParameterInfo> mParameters;
   private ArrayList<ClassInfo> mThrownExceptions;
   private String[] mParamStrings;
@@ -727,8 +793,8 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
   private AnnotationValueInfo mDefaultAnnotationElementValue;
   private String mReasonOpened;
   private ArrayList<Resolution> mResolutions;
-  
-  // TODO: merge with droiddoc version (above)  
+
+  // TODO: merge with droiddoc version (above)
   public String qualifiedName() {
     String parentQName = (containingClass() != null)
         ? (containingClass().qualifiedName() + ".") : "";
@@ -747,7 +813,7 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
         }
         params.append(pInfo.type().fullName());
       }
-      
+
       params.append(")");
       mSignature = params.toString();
     }
@@ -769,23 +835,37 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
 
   public boolean isConsistent(MethodInfo mInfo) {
     boolean consistent = true;
-    if (this.mReturnType != mInfo.mReturnType && !this.mReturnType.equals(mInfo.mReturnType)) {
-      if (!mReturnType.isPrimitive() && !mInfo.mReturnType.isPrimitive()) {
-        // Check to see if our class extends the old class.
-        ApiInfo infoApi = mInfo.containingClass().containingPackage().containingApi();
-        ClassInfo infoReturnClass = infoApi.findClass(mInfo.mReturnType.qualifiedTypeName());
-        // Find the classes.
-        consistent = infoReturnClass != null &&
-                     infoReturnClass.isAssignableTo(mReturnType.qualifiedTypeName());
-      } else {
+    if (!mReturnType.isTypeVariable() && !mInfo.mReturnType.isTypeVariable()) {
+      if (!mReturnType.equals(mInfo.mReturnType) ||
+          mReturnType.dimension() != mInfo.mReturnType.dimension()) {
         consistent = false;
       }
-
-      if (!consistent) {
-        Errors.error(Errors.CHANGED_TYPE, mInfo.position(), "Method "
-            + mInfo.prettyQualifiedSignature() + " has changed return type from " + mReturnType
-            + " to " + mInfo.mReturnType);
+    } else if (!mReturnType.isTypeVariable() && mInfo.mReturnType.isTypeVariable()) {
+      List<ClassInfo> constraints = mInfo.resolveConstraints(mInfo.mReturnType);
+      for (ClassInfo constraint : constraints) {
+        if (!constraint.isAssignableTo(mReturnType.qualifiedTypeName())) {
+          consistent = false;
+        }
       }
+    } else if (mReturnType.isTypeVariable() && !mInfo.mReturnType.isTypeVariable()) {
+      // It's never valid to go from being a parameterized type to not being one.
+      // This would drop the implicit cast breaking backwards compatibility.
+      consistent = false;
+    } else {
+      // If both return types are parameterized then the constraints must be
+      // exactly the same.
+      List<ClassInfo> currentConstraints = mInfo.resolveConstraints(mReturnType);
+      List<ClassInfo> newConstraints = mInfo.resolveConstraints(mInfo.mReturnType);
+      if (currentConstraints.size() != newConstraints.size() ||
+          currentConstraints.retainAll(newConstraints)) {
+        consistent = false;
+      }
+    }
+
+    if (!consistent) {
+      Errors.error(Errors.CHANGED_TYPE, mInfo.position(), "Method "
+          + mInfo.prettyQualifiedSignature() + " has changed return type from " + mReturnType
+          + " to " + mInfo.mReturnType);
     }
 
     if (mIsAbstract != mInfo.mIsAbstract) {
@@ -874,6 +954,43 @@ public class MethodInfo extends MemberInfo implements AbstractMethodInfo, Resolv
     return consistent;
   }
 
+  public TypeInfo getTypeParameter(String qualifiedTypeName) {
+    if (hasTypeParameters()) {
+      for (TypeInfo parameter : mTypeParameters) {
+        if (parameter.qualifiedTypeName().equals(qualifiedTypeName)) {
+          return parameter;
+        }
+      }
+    }
+    return containingClass().getTypeParameter(qualifiedTypeName);
+  }
+
+  // Given a type parameter it returns a list of all of the classes and interfaces it must extend
+  // and implement.
+  private List<ClassInfo> resolveConstraints(TypeInfo type) {
+    ApiInfo api = containingClass().containingPackage().containingApi();
+    List<ClassInfo> classes = new ArrayList<>();
+    Queue<TypeInfo> types = new LinkedList<>();
+    types.add(type);
+    while (!types.isEmpty()) {
+      type = types.poll();
+      if (!type.isTypeVariable()) {
+        ClassInfo cl = api.findClass(type.qualifiedTypeName());
+        if (cl != null) {
+          classes.add(cl);
+        }
+      } else {
+        TypeInfo parameter = getTypeParameter(type.qualifiedTypeName());
+        if (parameter.extendsBounds() != null) {
+          for (TypeInfo bound : parameter.extendsBounds()) {
+            types.add(bound);
+          }
+        }
+      }
+    }
+    return classes;
+  }
+
   public void printResolutions() {
       if (mResolutions == null || mResolutions.isEmpty()) {
           return;
