@@ -92,9 +92,9 @@ public class PageMetadata {
   *
   * @param rootTypeNodesList A list of root metadata nodes, each
   *        representing a type and it's member child pages.
+  * @deprecated
   */
   public static void WriteList(List<Node> rootTypeNodesList) {
-
     Collections.sort(rootTypeNodesList, BY_TYPE_NAME);
     Node pageMeta = new Node.Builder().setLabel("TOP").setChildren(rootTypeNodesList).build();
 
@@ -105,7 +105,77 @@ public class PageMetadata {
     // write the taglist to js file
     Data data = Doclava.makeHDF();
     data.setValue("reference_tree", buf.toString());
-    ClearPage.write(data, "jd_lists_unified.cs", "jd_lists_unified.js");
+    ClearPage.write(data, "jd_lists_unified.cs",  "jd_lists_unified.js");
+  }
+
+  /**
+  * Given a list of metadata nodes organized by lang, sort the
+  * root nodes by type name and render the types and their child
+  * metadata nodes to separate lang-specific json files in the out dir.
+  *
+  * @param rootNodesList A list of root metadata nodes, each
+  *        representing a type and it's member child pages.
+  */
+  public static void WriteListByLang(List<Node> rootNodesList) {
+    Collections.sort(rootNodesList, BY_LANG_NAME);
+    for (Node n : rootNodesList) {
+      String langFilename = "";
+      String langname = n.getLang();
+      langFilename = "_" + langname;
+      Collections.sort(n.getChildren(), BY_TYPE_NAME);
+      Node pageMeta = new Node.Builder().setLabel("TOP").setChildren(n.getChildren()).build();
+
+      StringBuilder buf = new StringBuilder();
+      // write the taglist to string format
+      pageMeta.renderLangResources(buf,langname);
+      //pageMeta.renderTypesByTag(buf);
+      // write the taglist to js file
+      Data data = Doclava.makeHDF();
+      data.setValue("reference_tree", buf.toString());
+      data.setValue("metadata.lang", langname);
+      String unifiedFilename = "jd_lists_unified" + langFilename + ".js";
+      String extrasFilename = "jd_extras" + langFilename + ".js";
+      // write out jd_lists_unified for each lang
+      ClearPage.write(data, "jd_lists_unified.cs", unifiedFilename);
+      // append jd_extras to jd_lists_unified for each lang, then delete.
+      appendExtrasMetadata(extrasFilename, unifiedFilename);
+    }
+  }
+
+  /**
+  * Given a list of metadata nodes organized by lang, sort the
+  * root nodes by type name and render the types and their child
+  * samples metadata nodes only to separate lang-specific json files
+  * in the out dir. Only used by devsite (ds) builds.
+  *
+  * @param rootNodesList A list of root metadata nodes, each
+  *        representing a type and it's member child pages.
+  */
+  public static void WriteSamplesListByLang(List<Node> rootNodesList) {
+    Collections.sort(rootNodesList, BY_LANG_NAME);
+    for (Node n : rootNodesList) {
+      boolean langHasSamples = false;
+      String langFilename = "";
+      String langname = n.getLang();
+      langFilename = "_" + langname;
+      Collections.sort(n.getChildren(), BY_TYPE_NAME);
+      Node pageMeta = new Node.Builder().setLabel("TOP").setChildren(n.getChildren()).build();
+
+      StringBuilder buf = new StringBuilder();
+      // write the taglist to string format
+      langHasSamples = pageMeta.renderSamplesResources(buf,langname);
+      // write the taglist to js file
+      Data data = Doclava.makeHDF();
+      data.setValue("reference_tree", buf.toString());
+      data.setValue("metadata.lang", langname);
+
+      if (langHasSamples) {
+        data.setValue("samples_only", "1");
+        // write out jd_lists_unified for each lang
+        String unifiedFilename = "android_samples_metadata" + langFilename + ".js";
+        ClearPage.write(data, "jd_lists_unified.cs", unifiedFilename);
+      }
+    }
   }
 
   /**
@@ -137,16 +207,17 @@ public class PageMetadata {
     if (!excludeNode) {
       Node pageMeta = new Node.Builder().build();
       pageMeta.setLabel(getTitleNormalized(hdf, "page.title"));
-      pageMeta.setTitleFriendly(hdf.getValue("page.titleFriendly",""));
+      pageMeta.setCategory(hdf.getValue("page.category",""));
       pageMeta.setSummary(hdf.getValue("page.metaDescription",""));
       pageMeta.setLink(getPageUrlNormalized(filename));
       pageMeta.setGroup(getStringValueNormalized(hdf,"sample.group"));
       pageMeta.setKeywords(getPageTagsNormalized(hdf, "page.tags"));
       pageMeta.setTags(getPageTagsNormalized(hdf, "meta.tags"));
       pageMeta.setImage(getImageUrlNormalized(hdf.getValue("page.image", "")));
-      pageMeta.setLang(getLangStringNormalized(filename));
+      pageMeta.setLang(getLangStringNormalized(hdf, filename));
       pageMeta.setType(getStringValueNormalized(hdf, "page.type"));
-      appendMetaNodeByType(pageMeta, tagList);
+      pageMeta.setTimestamp(hdf.getValue("page.timestamp",""));
+      appendMetaNodeByLang(pageMeta, tagList);
     }
   }
 
@@ -228,6 +299,7 @@ public class PageMetadata {
             XPathConstants.NODESET);
         if (imgNodes.getLength() > 0) {
           imageUrl = imgNodes.item(0).getNodeValue();
+          imageUrl = getImageUrlNormalized(imageUrl);
           imgFrom = "itemprop";
         } else {
           XPathExpression FirstImgExpr = xpath.compile("//img/@src");
@@ -323,7 +395,9 @@ public class PageMetadata {
     StringBuilder outString =  new StringBuilder();
     String tagList = hdf.getValue(tag, "");
     tagList.replaceAll("\"", "");
-    if (!tagList.isEmpty()) {
+    if ("".equals(tagList)) {
+      return tagList;
+    } else {
       int end = tagList.indexOf(",");
       if (end != -1) {
         tagList = tagList.substring(0,end);
@@ -333,8 +407,8 @@ public class PageMetadata {
         tagList = tagList.toLowerCase();
       }
       outString.append(tagList.trim());
+      return outString.toString();
     }
-    return outString.toString();
   }
 
   /**
@@ -370,19 +444,24 @@ public class PageMetadata {
   * @param filename A path string to the file relative to root.
   * @return A normalized lang value.
   */
-  public static String getLangStringNormalized(String filename) {
-    String[] stripStr = filename.toLowerCase().split("\\/");
+  public static String getLangStringNormalized(Data data, String filename) {
+    String[] stripStr = filename.toLowerCase().split("\\/", 3);
     String outFrag = "en";
+    String pathCanonical = filename;
     if (stripStr.length > 0) {
       for (String t : DocFile.DEVSITE_VALID_LANGS) {
         if ("intl".equals(stripStr[0])) {
           if (t.equals(stripStr[1])) {
             outFrag = stripStr[1];
+            //extract the root url (exclusive of intl/nn)
+            pathCanonical = stripStr[2];
             break;
           }
         }
       }
     }
+    //extract the root url (exclusive of intl/nn)
+    data.setValue("path.canonical", pathCanonical);
     return outFrag;
   }
 
@@ -472,13 +551,48 @@ public class PageMetadata {
   */
   public static String getPageUrlNormalized(String url) {
     String absUrl = "";
+
     if ((url !=null) && (!url.equals(""))) {
       absUrl = url.replace("{@docRoot}", "");
+      if (Doclava.USE_DEVSITE_LOCALE_OUTPUT_PATHS) {
+        absUrl = absUrl.replaceFirst("^en/", "");
+      }
       absUrl = absUrl.replaceFirst("^/(?!/)", "");
     }
     return absUrl;
   }
 
+  /**
+  * Given a metadata node, add it as a child of a root node based on its
+  * type. If there is no root node that matches the node's type, create one
+  * and add the metadata node as a child node.
+  *
+  * @param gNode The node to attach to a root node or add as a new root node.
+  * @param rootList The current list of root nodes.
+  * @return The updated list of root nodes.
+  */
+  public static List<Node> appendMetaNodeByLang(Node gNode, List<Node> rootList) {
+
+    String nodeLang = gNode.getLang();
+    boolean matched = false;
+    for (Node n : rootList) {
+      if (n.getLang().equals(nodeLang)) {  //find any matching lang node
+        appendMetaNodeByType(gNode,n.getChildren());
+        //n.getChildren().add(gNode);
+        matched = true;
+        break; // add to the first root node only
+      } // tag did not match
+    } // end rootnodes matching iterator
+    if (!matched) {
+      List<Node> mlangList = new ArrayList<Node>(); // list of file objects that have a given lang
+      //mlangList.add(gNode);
+      Node tnode = new Node.Builder().setChildren(mlangList).setLang(nodeLang).build();
+      rootList.add(tnode);
+      appendMetaNodeByType(gNode, mlangList);
+    }
+    return rootList;
+  }
+
   /**
   * Given a metadata node, add it as a child of a root node based on its
   * type. If there is no root node that matches the node's type, create one
@@ -543,6 +657,20 @@ public class PageMetadata {
     return rootTagNodesList;
   }
 
+  /**
+  * Append the contents of jd_extras to jd_lists_unified for each language.
+  *
+  * @param extrasFilename The lang-specific extras file to append.
+  * @param unifiedFilename The lang-specific unified metadata file.
+  */
+  public static void appendExtrasMetadata (String extrasFilename, String unifiedFilename) {
+
+    File f = new File(ClearPage.outputDir + "/" + extrasFilename);
+    if (f.exists() && !f.isDirectory()) {
+      ClearPage.copyFile(true, f, unifiedFilename, true);
+    }
+  }
+
   public static final Comparator<Node> BY_TAG_NAME = new Comparator<Node>() {
     public int compare (Node one, Node other) {
       return one.getLabel().compareTo(other.getLabel());
@@ -555,13 +683,19 @@ public class PageMetadata {
     }
   };
 
+    public static final Comparator<Node> BY_LANG_NAME = new Comparator<Node>() {
+    public int compare (Node one, Node other) {
+      return one.getLang().compareTo(other.getLang());
+    }
+  };
+
   /**
   * A node for storing page metadata. Use Builder.build() to instantiate.
   */
   public static class Node {
 
     private String mLabel; // holds page.title or similar identifier
-    private String mTitleFriendly; // title for card or similar use
+    private String mCategory; // subtabs, example 'training' 'guides'
     private String mSummary; // Summary for card or similar use
     private String mLink; //link href for item click
     private String mGroup; // from sample.group in _index.jd
@@ -570,11 +704,12 @@ public class PageMetadata {
     private String mImage; // holds an href, fully qualified or relative to root
     private List<Node> mChildren;
     private String mLang;
-    private String mType; // can be file, dir, video show, announcement, etc.
+    private String mType; // design, develop, distribute, youtube, blog, etc
+    private String mTimestamp; // optional timestamp eg 1447452827
 
     private Node(Builder builder) {
       mLabel = builder.mLabel;
-      mTitleFriendly = builder.mTitleFriendly;
+      mCategory = builder.mCategory;
       mSummary = builder.mSummary;
       mLink = builder.mLink;
       mGroup = builder.mGroup;
@@ -584,16 +719,17 @@ public class PageMetadata {
       mChildren = builder.mChildren;
       mLang = builder.mLang;
       mType = builder.mType;
+      mTimestamp = builder.mTimestamp;
     }
 
     private static class Builder {
-      private String mLabel, mTitleFriendly, mSummary, mLink, mGroup, mImage, mLang, mType;
+      private String mLabel, mCategory, mSummary, mLink, mGroup, mImage, mLang, mType, mTimestamp;
       private List<String> mKeywords = null;
       private List<String> mTags = null;
       private List<Node> mChildren = null;
       public Builder setLabel(String mLabel) { this.mLabel = mLabel; return this;}
-      public Builder setTitleFriendly(String mTitleFriendly) {
-        this.mTitleFriendly = mTitleFriendly; return this;
+      public Builder setCategory(String mCategory) {
+        this.mCategory = mCategory; return this;
       }
       public Builder setSummary(String mSummary) {this.mSummary = mSummary; return this;}
       public Builder setLink(String mLink) {this.mLink = mLink; return this;}
@@ -606,6 +742,7 @@ public class PageMetadata {
       public Builder setChildren(List<Node> mChildren) {this.mChildren = mChildren; return this;}
       public Builder setLang(String mLang) {this.mLang = mLang; return this;}
       public Builder setType(String mType) {this.mType = mType; return this;}
+      public Builder setTimestamp(String mTimestamp) {this.mTimestamp = mTimestamp; return this;}
       public Node build() {return new Node(this);}
     }
 
@@ -626,6 +763,49 @@ public class PageMetadata {
         }
       }
     }
+
+    /**
+    * Render a tree of metadata nodes organized by lang.
+    * @param buf Output buffer to render to.
+    */
+    void renderLangResources(StringBuilder buf, String langname) {
+      List<Node> list = mChildren; //list of type rootnodes
+      if (list == null || list.size() == 0) {
+        buf.append("null");
+      } else {
+        final int n = list.size();
+        for (int i = 0; i < n; i++) {
+          buf.append("METADATA['" + langname + "']." + list.get(i).mType + " = [");
+          list.get(i).renderTypes(buf); //render this lang's children
+          buf.append("\n];\n\n");
+        }
+      }
+    }
+
+    /**
+    * Render a tree of metadata nodes of type 'develop' to extract
+    * samples metadata. Only used by devsite (ds) builds.
+    * @param buf Output buffer to render to.
+    * @return true if samples were rendered to buf
+    */
+    boolean renderSamplesResources(StringBuilder buf, String langname) {
+      boolean langHasSamples = false;
+      List<Node> list = mChildren; //list of type rootnodes
+      if (list == null || list.size() == 0) {
+        buf.append("null");
+      } else {
+        final int n = list.size();
+        for (int i = 0; i < n; i++) {
+          //samples are always in type 'develop', so restrict
+          if ("develop".equals(list.get(i).mType)) {
+            //render this type's children
+            langHasSamples = list.get(i).renderTypeForSamples(buf);
+          }
+        }
+      }
+      return langHasSamples;
+    }
+
     /**
     * Render all metadata nodes for a specific type.
     * @param buf Output buffer to render to.
@@ -638,16 +818,33 @@ public class PageMetadata {
         final int n = list.size();
         for (int i = 0; i < n; i++) {
           buf.append("\n      {\n");
-          buf.append("        \"title\":\"" + list.get(i).mLabel + "\",\n" );
-          buf.append("        \"titleFriendly\":\"" + list.get(i).mTitleFriendly + "\",\n" );
-          buf.append("        \"summary\":\"" + list.get(i).mSummary + "\",\n" );
+          buf.append("        \"title\":\"");
+          renderStrWithUcs(buf, list.get(i).mLabel);
+          buf.append("\",\n" );
+          buf.append("        \"summary\":\"");
+          renderStrWithUcs(buf, list.get(i).mSummary);
+          buf.append("\",\n" );
           buf.append("        \"url\":\"" + list.get(i).mLink + "\",\n" );
-          buf.append("        \"group\":\"" + list.get(i).mGroup + "\",\n" );
+          if (!"".equals(list.get(i).mImage)) {
+            buf.append("        \"image\":\"" + list.get(i).mImage + "\",\n" );
+          }
+          if (!"".equals(list.get(i).mGroup)) {
+            buf.append("        \"group\":\"");
+            renderStrWithUcs(buf, list.get(i).mGroup);
+            buf.append("\",\n" );
+          }
+          if (!"".equals(list.get(i).mCategory)) {
+            buf.append("        \"category\":\"" + list.get(i).mCategory + "\",\n" );
+          }
+          if ((list.get(i).mType != null) && (list.get(i).mType != "")) {
+            buf.append("        \"type\":\"" + list.get(i).mType + "\",\n");
+          }
           list.get(i).renderArrayType(buf, list.get(i).mKeywords, "keywords");
           list.get(i).renderArrayType(buf, list.get(i).mTags, "tags");
-          buf.append("        \"image\":\"" + list.get(i).mImage + "\",\n" );
-          buf.append("        \"lang\":\"" + list.get(i).mLang + "\",\n" );
-          buf.append("        \"type\":\"" + list.get(i).mType + "\"");
+          if (!"".equals(list.get(i).mTimestamp)) {
+            buf.append("        \"timestamp\":\"" + list.get(i).mTimestamp + "\",\n");
+          }
+          buf.append("        \"lang\":\"" + list.get(i).mLang + "\"" );
           buf.append("\n      }");
           if (i != n - 1) {
             buf.append(", ");
@@ -656,6 +853,61 @@ public class PageMetadata {
       }
     }
 
+    /**
+    * Render all metadata nodes for samples only.
+    * Only used by devsite (ds) builds.
+    * @param buf Output buffer to render to.
+    * @return whether any samples were rendered to buf
+    */
+    boolean renderTypeForSamples(StringBuilder buf) {
+      boolean typeHasSamples = false;
+      List<Node> list = mChildren;
+      if (list == null || list.size() == 0) {
+        buf.append("nulltype");
+      } else {
+        final int n = list.size();
+        for (int i = 0; i < n; i++) {
+          // valid samples must have category 'samples'
+          if ("samples".equals(list.get(i).mCategory)) {
+            typeHasSamples = true;
+            buf.append("\n      {\n");
+            buf.append("        \"title\":\"");
+            renderStrWithUcs(buf, list.get(i).mLabel);
+            buf.append("\",\n" );
+            buf.append("        \"summary\":\"");
+            renderStrWithUcs(buf, list.get(i).mSummary);
+            buf.append("\",\n" );
+            buf.append("        \"url\":\"" + list.get(i).mLink + "\",\n" );
+            if (!"".equals(list.get(i).mImage)) {
+              buf.append("        \"image\":\"" + list.get(i).mImage + "\",\n" );
+            }
+            if (!"".equals(list.get(i).mGroup)) {
+              buf.append("        \"group\":\"");
+              renderStrWithUcs(buf, list.get(i).mGroup);
+              buf.append("\",\n" );
+            }
+            if (!"".equals(list.get(i).mCategory)) {
+              buf.append("        \"category\":\"" + list.get(i).mCategory + "\",\n" );
+            }
+            if ((list.get(i).mType != null) && (list.get(i).mType != "")) {
+              buf.append("        \"type\":\"" + list.get(i).mType + "\",\n");
+            }
+            list.get(i).renderArrayType(buf, list.get(i).mKeywords, "keywords");
+            list.get(i).renderArrayType(buf, list.get(i).mTags, "tags");
+            if (!"".equals(list.get(i).mTimestamp)) {
+              buf.append("        \"timestamp\":\"" + list.get(i).mTimestamp + "\",\n");
+            }
+            buf.append("        \"lang\":\"" + list.get(i).mLang + "\"" );
+            buf.append("\n      }");
+            if (i != n - 1) {
+              buf.append(", ");
+            }
+          }
+        }
+      }
+      return typeHasSamples;
+    }
+
     /**
     * Build and render a list of tags associated with each type.
     * @param buf Output buffer to render to.
@@ -722,22 +974,7 @@ public class PageMetadata {
         final int n = list.size();
         for (int i = 0; i < n; i++) {
           String tagval = list.get(i).toString();
-          final int L = tagval.length();
-          for (int t = 0; t < L; t++) {
-            char c = tagval.charAt(t);
-            if (c >= Character.MIN_HIGH_SURROGATE && c <= Character.MAX_HIGH_SURROGATE ) {
-              // we have a UTF-16 multi-byte character
-              int codePoint = tagval.codePointAt(t);
-              int charSize = Character.charCount(codePoint);
-              t += charSize - 1;
-              buf.append(String.format("\\u%04x",codePoint));
-            } else if (c >= ' ' && c <= '~' && c != '\\') {
-              buf.append(c);
-            } else { 
-              // we are encoding a two byte character
-              buf.append(String.format("\\u%04x", (int) c));
-            }
-          }
+          renderStrWithUcs(buf,tagval);
           if (i != n - 1) {
             buf.append(",");
           }
@@ -745,6 +982,31 @@ public class PageMetadata {
       }
     }
 
+    /**
+    * Render a string that can include ucs2 encoded characters.
+    * @param buf Output buffer to render to.
+    * @param chars String to append to buf with any necessary encoding
+    */
+    void renderStrWithUcs(StringBuilder buf, String chars) {
+      String strval = chars;
+      final int L = strval.length();
+      for (int t = 0; t < L; t++) {
+        char c = strval.charAt(t);
+        if (c >= Character.MIN_HIGH_SURROGATE && c <= Character.MAX_HIGH_SURROGATE ) {
+          // we have a UTF-16 multi-byte character
+          int codePoint = strval.codePointAt(t);
+          int charSize = Character.charCount(codePoint);
+          t += charSize - 1;
+          buf.append(String.format("\\u%04x",codePoint));
+        } else if (c >= ' ' && c <= '~' && c != '\\') {
+          buf.append(c);
+        } else {
+          // we are encoding a two byte character
+          buf.append(String.format("\\u%04x", (int) c));
+        }
+      }
+    }
+
     public String getLabel() {
       return mLabel;
     }
@@ -753,12 +1015,12 @@ public class PageMetadata {
        mLabel = label;
     }
 
-    public String getTitleFriendly() {
-      return mTitleFriendly;
+    public String getCategory() {
+      return mCategory;
     }
 
-    public void setTitleFriendly(String title) {
-       mTitleFriendly = title;
+    public void setCategory(String title) {
+       mCategory = title;
     }
 
     public String getSummary() {
@@ -849,8 +1111,16 @@ public class PageMetadata {
       return mType;
     }
 
+    public String getTimestamp() {
+      return mTimestamp;
+    }
+
     public void setType(String type) {
       mType = type;
     }
+
+    public void setTimestamp(String timestamp) {
+      mTimestamp = timestamp;
+    }
   }
 }
