@@ -24,7 +24,7 @@ public class TypeInfo implements Resolvable {
   public static final Set<String> PRIMITIVE_TYPES = Collections.unmodifiableSet(
       new HashSet<String>(Arrays.asList("boolean", "byte", "char", "double", "float", "int",
       "long", "short", "void")));
-  
+
   public TypeInfo(boolean isPrimitive, String dimension, String simpleTypeName,
       String qualifiedTypeName, ClassInfo cl) {
     mIsPrimitive = isPrimitive;
@@ -39,16 +39,17 @@ public class TypeInfo implements Resolvable {
     if (typeString.endsWith("...")) {
       typeString = typeString.substring(0, typeString.length() - 3);
     }
-    
+
     // Generic parameters
+    int extendsPos = typeString.indexOf(" extends ");
     int paramStartPos = typeString.indexOf('<');
-    if (paramStartPos > -1) {
+    if (paramStartPos > -1 && (extendsPos == -1 || paramStartPos < extendsPos)) {
       ArrayList<TypeInfo> generics = new ArrayList<TypeInfo>();
-      int paramEndPos = typeString.lastIndexOf('>');
-      
+      int paramEndPos = 0;
+
       int entryStartPos = paramStartPos + 1;
       int bracketNesting = 0;
-      for (int i = entryStartPos; i < paramEndPos; i++) {
+      for (int i = entryStartPos; i < typeString.length(); i++) {
         char c = typeString.charAt(i);
         if (c == ',' && bracketNesting == 0) {
           String entry = typeString.substring(entryStartPos, i).trim();
@@ -59,30 +60,61 @@ public class TypeInfo implements Resolvable {
           bracketNesting++;
         } else if (c == '>') {
           bracketNesting--;
+          // Once bracketNesting goes negative, we've found the closing angle bracket
+          if (bracketNesting < 0) {
+            paramEndPos = i;
+            break;
+          }
         }
       }
-     
+
       TypeInfo info = new TypeInfo(typeString.substring(entryStartPos, paramEndPos).trim());
       generics.add(info);
-      
+      addResolution(new Resolution("variability", "", null));
+
       mTypeArguments = generics;
-      
+
       if (paramEndPos < typeString.length() - 1) {
         typeString = typeString.substring(0,paramStartPos) + typeString.substring(paramEndPos + 1);
       } else {
         typeString = typeString.substring(0,paramStartPos);
       }
     }
-    
-    // Dimensions
-    int pos = typeString.indexOf('['); 
+
+    // The previous extends may have been within the generic type parameters which we don't
+    // actually care about and were removed from the type string above
+    extendsPos = typeString.indexOf(" extends ");
+    if (extendsPos > -1) {
+      ArrayList<TypeInfo> extendsBounds = new ArrayList<>();
+      int entryStartPos = extendsPos + 9;
+      int bracketNesting = 0;
+      for (int i = entryStartPos; i < typeString.length(); i++) {
+        char c = typeString.charAt(i);
+        if (c == '&' && bracketNesting == 0) {
+          String entry = typeString.substring(entryStartPos, i).trim();
+          TypeInfo info = new TypeInfo(entry);
+          extendsBounds.add(info);
+          entryStartPos = i + 1;
+        } else if (c == '<') {
+          bracketNesting++;
+        } else if (c == '>') {
+          bracketNesting--;
+        }
+      }
+      TypeInfo info = new TypeInfo(typeString.substring(entryStartPos, typeString.length()).trim());
+      extendsBounds.add(info);
+      mExtendsBounds = extendsBounds;
+      typeString = typeString.substring(0, extendsPos);
+    }
+
+    int pos = typeString.indexOf('[');
     if (pos > -1) {
       mDimension = typeString.substring(pos);
       typeString = typeString.substring(0, pos);
     } else {
       mDimension = "";
     }
-   
+
     if (PRIMITIVE_TYPES.contains(typeString)) {
       mIsPrimitive = true;
       mSimpleTypeName = typeString;
@@ -121,7 +153,17 @@ public class TypeInfo implements Resolvable {
     mFullName = other.fullName();
   }
 
+  /**
+   * Returns this type as a {@link ClassInfo} if it represents a class or
+   * interface.
+   */
   public ClassInfo asClassInfo() {
+    if (!mResolvedClass) {
+      mResolvedClass = true;
+      if (mClass == null && !mIsPrimitive && !mIsTypeVariable && !mIsWildcard) {
+        mClass = Converter.obtainClass(qualifiedTypeName());
+      }
+    }
     return mClass;
   }
 
@@ -232,6 +274,17 @@ public class TypeInfo implements Resolvable {
     return fullName;
   }
 
+  public String dexName() {
+    if (mIsTypeVariable || mIsWildcard) {
+      if (mExtendsBounds != null && !mExtendsBounds.isEmpty()) {
+        return mExtendsBounds.get(0).dexName() + mDimension;
+      } else {
+        return "java.lang.Object" + mDimension;
+      }
+    }
+    return mQualifiedTypeName + mDimension;
+  }
+
   public ArrayList<TypeInfo> typeArguments() {
     return mTypeArguments;
   }
@@ -317,7 +370,7 @@ public class TypeInfo implements Resolvable {
       mTypeArguments.add(arg);
   }
 
-  void setBounds(ArrayList<TypeInfo> superBounds, ArrayList<TypeInfo> extendsBounds) {
+  public void setBounds(ArrayList<TypeInfo> superBounds, ArrayList<TypeInfo> extendsBounds) {
     mSuperBounds = superBounds;
     mExtendsBounds = extendsBounds;
   }
@@ -330,7 +383,7 @@ public class TypeInfo implements Resolvable {
       return mExtendsBounds;
   }
 
-  void setIsTypeVariable(boolean b) {
+  public void setIsTypeVariable(boolean b) {
     mIsTypeVariable = b;
   }
 
@@ -342,7 +395,7 @@ public class TypeInfo implements Resolvable {
       return mIsWildcard;
   }
 
-  static HashSet<String> typeVariables(ArrayList<TypeInfo> params) {
+  public static HashSet<String> typeVariables(ArrayList<TypeInfo> params) {
     return typeVariables(params, new HashSet<String>());
   }
 
@@ -362,6 +415,16 @@ public class TypeInfo implements Resolvable {
     return mIsTypeVariable;
   }
 
+  public void resolveTypeVariables(HashSet<String> variables) {
+    if (mExtendsBounds != null) {
+      for (TypeInfo bound : mExtendsBounds) {
+        if (variables.contains(bound.qualifiedTypeName())) {
+          bound.setIsTypeVariable(true);
+        }
+      }
+    }
+  }
+
   public String defaultValue() {
     if (mIsPrimitive) {
       if ("boolean".equals(mSimpleTypeName)) {
@@ -440,6 +503,13 @@ public class TypeInfo implements Resolvable {
               } else {
                   mClass = InfoBuilder.Caches.obtainClass(qualifiedClassName.toString());
               }
+          } else if ("variability".equals(resolution.getVariable())) {
+              StringBuilder qualifiedClassName = new StringBuilder();
+              for (TypeInfo arg : mTypeArguments) {
+                InfoBuilder.resolveQualifiedName(arg.simpleTypeName(), qualifiedClassName,
+                        resolution.getInfoBuilder());
+                arg.setIsTypeVariable(!("".equals(qualifiedClassName.toString())));
+              }
           }
       }
 
@@ -475,9 +545,11 @@ public class TypeInfo implements Resolvable {
    */
   public static Map<String, TypeInfo> getTypeArgumentMapping(TypeInfo generic, TypeInfo typed) {
     Map<String, TypeInfo> map = new HashMap<String, TypeInfo>();
-    for (int i = 0; i < generic.typeArguments().size(); i++) {
-      if (typed.typeArguments() != null && typed.typeArguments().size() > i) {
-        map.put(generic.typeArguments().get(i).fullName(), typed.typeArguments().get(i));
+    if (generic != null && generic.typeArguments() != null) {
+      for (int i = 0; i < generic.typeArguments().size(); i++) {
+        if (typed.typeArguments() != null && typed.typeArguments().size() > i) {
+          map.put(generic.typeArguments().get(i).simpleTypeName(), typed.typeArguments().get(i));
+        }
       }
     }
     return map;
@@ -493,6 +565,9 @@ public class TypeInfo implements Resolvable {
 
   private ArrayList<Resolution> mResolutions;
 
+  /** Whether the value of {@code mClass} has been resolved. */
+  private boolean mResolvedClass;
+
   private boolean mIsPrimitive;
   private boolean mIsTypeVariable;
   private boolean mIsWildcard;
